@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -97,22 +97,6 @@ typedef enum
 }
 cdata_type;
 
-typedef struct gst_cfunc_descriptor
-{
-  OBJ_HEADER;
-  OOP cFunction;		/* gst_cobject whose C value is func
-				   addr */
-  OOP cFunctionName;		/* Name of C function in mapping table */
-  OOP returnType;		/* Smalltalk return type */
-  OOP numFixedArgs;		/* number of real arguments passed from
-				   smalltalk (excluding "self" parameters
-  				   which are synthetically added when
-				   calling the C function).  */
-  OOP argTypes[1];		/* variable length, really numFixedArgs 
-				   long */
-}
- *gst_cfunc_descriptor;
-
 typedef struct symbol_type_map
 {
   OOP *symbol;
@@ -331,6 +315,22 @@ get_errno (void)
   return (old);
 }
 
+int
+my_utime (const char *name,
+	  long new_atime,
+	  long new_mtime)
+{
+  struct timeval times[2];
+  int result;
+  times[0].tv_sec = new_atime + 86400 * 10957;
+  times[1].tv_sec = new_mtime + 86400 * 10957;
+  times[0].tv_usec = times[1].tv_usec = 0;
+  result = utimes (name, times);
+  if (!result)
+    errno = 0;
+  return (result);
+}
+
 int
 my_stat (const char *name,
 	 gst_stat * out)
@@ -523,7 +523,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("strerror", strerror);
   _gst_define_cfunc ("stat", my_stat);
   _gst_define_cfunc ("lstat", my_lstat);
-  _gst_define_cfunc ("utime", _gst_set_file_access_times);
+  _gst_define_cfunc ("utime", my_utime);
 
   _gst_define_cfunc ("opendir", my_opendir);
   _gst_define_cfunc ("closedir", closedir);
@@ -531,7 +531,6 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("rewinddir", rewinddir);
   _gst_define_cfunc ("extractDirentName", extract_dirent_name);
 
-  _gst_define_cfunc ("symlink", symlink);
   _gst_define_cfunc ("unlink", unlink);
   _gst_define_cfunc ("rename", rename);
   _gst_define_cfunc ("rmdir", rmdir);
@@ -629,6 +628,9 @@ _gst_invoke_croutine (OOP cFuncOOP,
   incPtr = INC_SAVE_POINTER ();
 
   desc = (gst_cfunc_descriptor) OOP_TO_OBJ (cFuncOOP);
+  if (IS_NIL (desc->cFunction))
+    return (NULL);
+
   c_func_cur = (cfunc_info *) COBJECT_VALUE (desc->cFunction);
   if (!c_func_cur)
     return (NULL);
@@ -1230,17 +1232,3 @@ _gst_set_errno(int errnum)
   _gst_errno = errnum;
 #endif
 }
-
-void
-_gst_restore_cfunc_descriptor (OOP cFuncDescOOP)
-{
-  gst_cfunc_descriptor desc;
-  cfunc_info *cfi;
-  char *funcName;
-
-  desc = (gst_cfunc_descriptor) OOP_TO_OBJ (cFuncDescOOP);
-  funcName = (char *) _gst_to_cstring (desc->cFunctionName);
-  cfi = lookup_function (funcName);
-  xfree (funcName);
-  SET_COBJECT_VALUE (desc->cFunction, cfi);
-}
