@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2005,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -69,15 +69,6 @@
    output for correctness.  */
 /* #define VERIFY_COMPILED_METHODS */
 
-/* Define this to inline #perform: with a constant selector.  It is
-   disabled because it removes some error checking.  For example, it
-   allows one to compile "self perform: #yourself with: 1", after
-   which the compiler thinks that the stack is balanced, when in
-   fact it is not (because #yourself is a SEND_IMMEDIATE selector);
-   or even worse, "self perform: #this:that: with: 1", where the stack
-   upon entry to #this:that: is confused beyond any recognition.  */
-/* #define INLINE_PERFORM */
-
 #define LITERAL_VEC_CHUNK_SIZE		32
 
 
@@ -107,6 +98,7 @@ mst_Boolean _gst_untrusted_methods = false;
    be placed into.  */
 OOP _gst_this_class = NULL;
 OOP _gst_this_category = NULL;
+static OOP this_method_category;
 
 /* This holds the gst_compiled_method oop for the most recently
    compiled method.  It is only really valid after a compile: has been
@@ -115,7 +107,7 @@ OOP _gst_latest_compiled_method = NULL;
 
 /* This flag controls whether byte codes are printed after
    compilation.  */
-mst_Boolean _gst_declare_tracing = false;
+int _gst_declare_tracing = 0;
 
 /* If true, the compilation of a set of methods will be skipped
    completely; only syntax will be checked.  Set by primitive, cleared
@@ -185,13 +177,6 @@ static mst_Boolean compile_to_by_do (tree_node to,
 				     tree_node by,
 				     tree_node block);
 
-/* Special case compilation of #perform: and friends with a constant
-   selector (in case of #perform:withArguments:, the arguments must be
-   a constant array or an array constructor).  This is done especially
-   for the Java translator, but is always good, so why not?  */
-static mst_Boolean compile_perform (OOP selectorOOP,
-				    tree_node expr);
-
 /* Special case compilation of a #and: or #or: boolean operation; very
    similar to compile_if_statement.  EXPR is a node for the entire
    keyword message send.  Returns true if byte codes were emitted,
@@ -230,10 +215,6 @@ static mst_Boolean compile_statements (tree_node statementList,
    them into a symbol OOP and returns that symbol.  */
 static OOP compute_selector (tree_node selectorExpr);
 
-/* Given a tree_node of type TREE_KEYWORD_LIST, pick out and concatenate
-   the keywords, turn them into a symbol OOP and return that symbol.  */
-static OOP compute_keyword_selector (tree_node selectorExpr);
-
 /* Given CONSTEXPR, a section of the syntax tree that represents a
    Smalltalk constant, this routine creates and returns an OOP to be
    stored as a method literal in the method that's currently being
@@ -246,8 +227,8 @@ static OOP make_constant_oop (tree_node constExpr);
    no literals for the current method, _gst_nil_oop is returned.  */
 static OOP get_literals_array (void);
 
-/* Process the attributes in ATTRIBUTELIST, return the primitive number
-   (so far, this is the only attribute we honor).  */
+/* Process the attributes in ATTRIBUTELIST, return the primitive number.
+   Also record a <category: ...> attribute in this_method_category.  */
 static int process_attributes_tree (tree_node attributeList);
 
 /* Process the attribute in MESSAGEOOP, return the primitive number
@@ -262,14 +243,16 @@ static OOP method_new (method_header header,
 		       OOP literals,
 		       bc_vector bytecodes,
 		       OOP class,
-		       OOP selector);
+		       OOP methodDesc);
 
 /* Returns an instance of MethodInfo.  This instance is used in the
    reconstruction of the source code for the method, and holds the
    category that the method belongs to.  */
 static OOP method_info_new (OOP class,
 			    OOP selector,
-			    method_attributes *attrs);
+			    method_attributes *attrs,
+			    OOP sourceCode,
+			    OOP categoryOOP);
 
 /* Returns a FileSegment instance for the currently open compilation
    stream.  FileSegment instances are used record information useful
@@ -278,8 +261,12 @@ static OOP method_info_new (OOP class,
    the instance variables are different; for a string, the entire
    contents of the string is preserved as the source code for the
    method; for a disk file loaded from within the kernel source
-   directory, the file name, byte offset and length are preserved.  */
-static OOP file_segment_new (void);
+   directory, the file name, byte offset and length are preserved.  
+   The two parameters are used in the latter case and they represent
+   respectively the first byte of the method and the first byte after
+   the method.  Otherwise they should be -1.  */
+static OOP file_segment_new (int64_t startPos,
+			     int64_t endPos);
 
 /* This creates a CompiledBlock for the given BYTECODES.  The bytecodes
    are passed through the peephole optimizer and stored, the header is
@@ -508,7 +495,8 @@ _gst_install_initial_methods (void)
   termination_method = _gst_make_new_method (0, 0, 0, 0, _gst_nil_oop,
 					     _gst_get_bytecodes (),
 					     _gst_this_class,
-					     _gst_terminate_symbol);
+					     _gst_terminate_symbol,
+					     _gst_this_category, -1, -1);
 
   install_method (termination_method);
 
@@ -546,7 +534,8 @@ _gst_invoke_hook (const char *hook)
 {
   int save_execution;
   save_execution = _gst_execution_tracing;
-  _gst_execution_tracing = _gst_trace_kernel_execution;
+  if (_gst_execution_tracing == 1)
+    _gst_execution_tracing = 0;
   _gst_msg_sendf (NULL, "%v %o changed: %S", 
 		  _gst_object_memory_class, hook);
   _gst_execution_tracing = save_execution;
@@ -615,8 +604,9 @@ _gst_display_compilation_trace (const char *string,
 
 
 OOP
-_gst_execute_statements (tree_node temporaries,
+_gst_execute_statements (tree_node temps,
 			 tree_node statements,
+			 enum undeclared_strategy undeclared,
 			 mst_Boolean quiet)
 {
   tree_node messagePattern;
@@ -627,12 +617,14 @@ _gst_execute_statements (tree_node temporaries,
 #endif
   OOP methodOOP;
   OOP oldClass, oldCategory;
+  enum undeclared_strategy oldUndeclared;
   inc_ptr incPtr;
+  YYLTYPE loc;
 
-  quiet = quiet || _gst_verbosity <= 1 || _gst_emacs_process
-	  || !_gst_get_cur_stream_prompt ();
-
-  quiet = quiet && !_gst_regression_testing;
+  if (_gst_regression_testing
+      || _gst_verbosity < 2
+      || !_gst_get_cur_stream_prompt ())
+    quiet = true;
 
   oldClass = _gst_this_class;
   oldCategory = _gst_this_category;
@@ -641,25 +633,31 @@ _gst_execute_statements (tree_node temporaries,
 
   _gst_set_compilation_class (_gst_undefined_object_class);
   _gst_set_compilation_category (_gst_nil_oop);
+  loc = _gst_get_location ();
 
   messagePattern = _gst_make_unary_expr (&statements->location,
-					 NULL,
-					 "executeStatements");
+					 NULL, "executeStatements");
 
   _gst_display_compilation_trace ("Compiling doit code", false);
 
   /* This is a big hack to let doits access the variables and classes
      in the current namespace.  */
+  oldUndeclared = _gst_set_undeclared (undeclared);
   SET_CLASS_ENVIRONMENT (_gst_undefined_object_class,
 			 _gst_current_namespace);
+
+  if (statements->nodeType != TREE_STATEMENT_LIST)
+    statements = _gst_make_statement_list (&statements->location, statements);
+
   methodOOP =
-    _gst_compile_method (_gst_make_method
-			 (&statements->location, messagePattern,
-			  temporaries, NULL, statements),
+    _gst_compile_method (_gst_make_method (&statements->location, &loc,
+					   messagePattern, temps, NULL,
+					   statements),
 			 true, false);
 
   SET_CLASS_ENVIRONMENT (_gst_undefined_object_class,
 			 _gst_smalltalk_dictionary);
+  _gst_set_undeclared (oldUndeclared);
 
   _gst_set_compilation_class (oldClass);
   _gst_set_compilation_category (oldCategory);
@@ -672,9 +670,6 @@ _gst_execute_statements (tree_node temporaries,
   incPtr = INC_SAVE_POINTER ();
   INC_ADD_OOP (methodOOP);
 
-  if (!quiet && (_gst_regression_testing || _gst_verbosity >= 3))
-    printf ("\nExecution begins...\n");
-
   _gst_bytecode_counter = _gst_primitives_executed =
     _gst_self_returns = _gst_inst_var_returns = _gst_literal_returns =
     0;
@@ -702,61 +697,48 @@ _gst_execute_statements (tree_node temporaries,
 
   INC_RESTORE_POINTER (incPtr);
 
-  if (quiet)
-    return (_gst_last_returned_value);
+  if (!quiet)
+    {
+      int save_execution;
 
-  /* Do more frequent flushing to ensure the result are well placed */
-  if (_gst_regression_testing)
-    fflush(stdout);
+      /* Do more frequent flushing to ensure the result are well placed */
+      if (_gst_verbosity >= 3)
+	{
+          printf ("returned value is ");
+          fflush(stdout);
+	}
 
-  if (_gst_regression_testing || _gst_verbosity >= 3)
-    printf ("returned value is ");
-
-  printf ("%O\n", _gst_last_returned_value);
-
-  if (_gst_regression_testing)
-    fflush(stdout);
-
-  if (_gst_verbosity < 3)
+      save_execution = _gst_execution_tracing;
+      if (_gst_execution_tracing == 1)
+        _gst_execution_tracing = 0;
+      _gst_str_msg_send (_gst_last_returned_value, "printNl", NULL);
+      _gst_execution_tracing = save_execution;
+    }
+  if (quiet || _gst_verbosity < 3)
     return (_gst_last_returned_value);
 
   deltaTime = endTime - startTime;
-  deltaTime += (deltaTime == 0);	/* it could be zero which would 
-					   core dump */
-
 #ifdef ENABLE_JIT_TRANSLATION
-#define GIVING_X_BYTECODES_PER_SEC
-#define BYTECODES_PER_SEC
-  printf ("Execution");
+  printf ("Execution took %.3f seconds", deltaTime / 1000.0);
 #else
-#define GIVING_X_BYTECODES_PER_SEC ", giving %lu bytecodes/sec"
-#define BYTECODES_PER_SEC , (unsigned long) (_gst_bytecode_counter/ (deltaTime / 1000.0))
-  printf ("%lu byte codes executed\nwhich", _gst_bytecode_counter);
+  printf ("%lu byte codes executed\nwhich took %.3f seconds",
+	  _gst_bytecode_counter, deltaTime / 1000.0);
 #endif
 
-  printf (" took %.3f seconds" GIVING_X_BYTECODES_PER_SEC "\n",
-	  deltaTime / 1000.0 BYTECODES_PER_SEC);
-#if 0 && defined(HAVE_GETRUSAGE)
+#ifdef HAVE_GETRUSAGE
   deltaTime = ((endRusage.ru_utime.tv_sec * 1000) +
 	       (endRusage.ru_utime.tv_usec / 1000)) -
     ((startRusage.ru_utime.tv_sec * 1000) +
      (startRusage.ru_utime.tv_usec / 1000));
-  deltaTime += (deltaTime == 0);	/* it could be zero which would 
-					   core dump */
-  printf ("(%.3f seconds user time" GIVING_X_BYTECODES_PER_SEC ", ",
-	  deltaTime / 1000.0 BYTECODES_PER_SEC);
+  printf (" (%.3fs user", deltaTime / 1000.0);
 
   deltaTime = ((endRusage.ru_stime.tv_sec * 1000) +
 	       (endRusage.ru_stime.tv_usec / 1000)) -
     ((startRusage.ru_stime.tv_sec * 1000) +
      (startRusage.ru_stime.tv_usec / 1000));
-  printf ("%.3f seconds system time)\n", deltaTime / 1000.0);
-
-  printf ("(%ld swaps, %ld minor page faults, %ld major page faults)\n",
-	  endRusage.ru_nswap - startRusage.ru_nswap,
-	  endRusage.ru_minflt - startRusage.ru_minflt,
-	  endRusage.ru_majflt - startRusage.ru_majflt);
+  printf ("+%.3fs sys)", deltaTime / 1000.0);
 #endif
+  printf ("\n");
 
 #ifdef ENABLE_JIT_TRANSLATION
   if (!_gst_sample_counter)
@@ -771,9 +753,8 @@ _gst_execute_statements (tree_node temporaries,
 
   printf ("%lu primitives, percent %.2f\n", _gst_primitives_executed,
 	  100.0 * _gst_primitives_executed / _gst_bytecode_counter);
-  printf
-    ("self returns %lu, inst var returns %lu, literal returns %lu\n",
-     _gst_self_returns, _gst_inst_var_returns, _gst_literal_returns);
+  printf ("self returns %lu, inst var returns %lu, literal returns %lu\n",
+          _gst_self_returns, _gst_inst_var_returns, _gst_literal_returns);
   printf ("%lu method cache lookups since last cleanup, percent %.2f\n",
 	  _gst_sample_counter,
 	  100.0 * _gst_sample_counter / _gst_bytecode_counter);
@@ -783,15 +764,12 @@ _gst_execute_statements (tree_node temporaries,
   printf ("%lu method cache hits, %lu misses", cacheHits,
 	  _gst_cache_misses);
   if (cacheHits || _gst_cache_misses)
-    printf (", %.2f percent hits\n",
-	    (100.0 * cacheHits) / _gst_sample_counter);
+    printf (", %.2f percent hits\n", (100.0 * cacheHits) / _gst_sample_counter);
   else
     printf ("\n");
 
   return (_gst_last_returned_value);
 }
-#undef GIVING_X_BYTECODES_PER_SEC
-#undef BYTECODES_PER_SEC
 
 
 
@@ -810,6 +788,7 @@ _gst_compile_method (tree_node method,
 
   dup_message_receiver = false;
   literal_vec_curr = literal_vec;
+  this_method_category = _gst_this_category;
   _gst_unregister_oop (_gst_latest_compiled_method);
   _gst_latest_compiled_method = _gst_nil_oop;
   _gst_line_number (-1, true);
@@ -909,7 +888,10 @@ _gst_compile_method (tree_node method,
 					_gst_get_arg_count (),
 					_gst_get_temp_count (),
 					stack_depth, _gst_nil_oop, bytecodes,
-					_gst_this_class, selector);
+					_gst_this_class, selector,
+					_gst_this_category,
+					method->location.file_offset,
+					method->v_method.endPos);
       INC_ADD_OOP (methodOOP);
 
       if (install)
@@ -1105,6 +1087,8 @@ compile_constant (tree_node constExpr)
 	      return;
 	    }
         }
+      else
+        MAKE_OOP_READONLY (constantOOP, true);
 
       index = add_literal (constantOOP);
     }
@@ -1405,15 +1389,6 @@ compile_keyword_expr (tree_node expr)
 			    expr->v_expr.expression->v_list.next->v_list.next->v_list.value))
 	return;
     }
-  else if (selector == _gst_perform_symbol
-	   || selector == _gst_perform_with_symbol
-	   || selector == _gst_perform_with_with_symbol
-	   || selector == _gst_perform_with_with_with_symbol
-	   || selector == _gst_perform_with_arguments_symbol)
-    {
-      if (compile_perform (selector, expr))
-	return;
-    }
 
   numArgs = list_length (expr->v_expr.expression);
 
@@ -1421,46 +1396,6 @@ compile_keyword_expr (tree_node expr)
   compile_send (expr, selector, numArgs);
 }
 
-mst_Boolean
-compile_perform (OOP selectorOOP,
-		 tree_node expr)
-{
-#ifdef INLINE_PERFORM
-  tree_node selector = expr->v_expr.expression->v_list.value;
-  tree_node args = expr->v_expr.expression->v_list.next;
-  int i;
-
-  /* We inline only if the selector is a symbol... */
-  if (selector->nodeType != TREE_CONST_EXPR
-      || selector->v_const.constType != CONST_OOP
-      || OOP_CLASS (selector->v_const.val.oopVal) != _gst_symbol_class)
-    return false;
-
-  /* ... and in case of perform:withArguments:, if the list of arguments
-     is a constant array or an array constructor.  */
-  if (selectorOOP == _gst_perform_with_arguments_symbol)
-    {
-      if (args->nodeType != TREE_ARRAY_CONSTRUCTOR
-          || !(args->nodeType == TREE_CONST_EXPR
-	       && args->v_const.constType == CONST_ARRAY))
-	return false;
-
-      /* dive into the list of members of the array. */
-      args = args->v_const.val.aVal;
-    }
-      
-  /* compile the values now */
-  for (i = 0; args; i++, args = args->v_list.next)
-    compile_statement (args->v_list.value);
-
-  /* and send the selector */
-  compile_send (expr, selector->v_const.val.oopVal, i);
-
-  return true;
-#else
-  return false;
-#endif
-}
 
 void
 compile_send (tree_node expr,
@@ -2117,6 +2052,11 @@ equal_constant (OOP oop,
 	return (true);
       break;
 
+    case CONST_BINDING:
+      if (oop == _gst_find_variable_binding (constExpr->v_const.val.aVal, false))
+	return (true);
+      break;
+
     case CONST_ARRAY:
       if (IS_OOP (oop) && OOP_CLASS (oop) == _gst_array_class)
 	{
@@ -2180,8 +2120,9 @@ make_constant_oop (tree_node constExpr)
 	  elementOOP = make_constant_oop (arrayElt->v_list.value);
 	  result = OOP_TO_OBJ (resultOOP);
 	  result->data[i] = elementOOP;
+	  if (IS_OOP (elementOOP))
+	    MAKE_OOP_READONLY (elementOOP, true);
 	}
-      MAKE_OOP_READONLY (resultOOP, true);
       INC_RESTORE_POINTER (incPtr);
       return (resultOOP);
     }
@@ -2204,9 +2145,7 @@ make_constant_oop (tree_node constExpr)
       return (floatq_new (constExpr->v_const.val.fVal));
 
     case CONST_STRING:
-      resultOOP = _gst_string_new (constExpr->v_const.val.sVal);
-      MAKE_OOP_READONLY (resultOOP, true);
-      return (resultOOP);
+      return (_gst_string_new (constExpr->v_const.val.sVal));
 
     case CONST_OOP:
       return (constExpr->v_const.val.oopVal);
@@ -2215,7 +2154,18 @@ make_constant_oop (tree_node constExpr)
       bo = constExpr->v_const.val.boVal;
       result = instantiate_with (bo->class, bo->size, &resultOOP);
       memcpy (result->data, bo->body, bo->size);
-      MAKE_OOP_READONLY (resultOOP, true);
+      return (resultOOP);
+
+    case CONST_BINDING:
+      resultOOP = _gst_find_variable_binding (constExpr->v_const.val.aVal,
+					      false);
+      if (IS_NIL (resultOOP))
+	{
+	  _gst_errorf_at (constExpr->location.first_line,
+			  "invalid variable binding");
+          EXIT_COMPILATION ();
+	}
+
       return (resultOOP);
 
     case CONST_ARRAY:
@@ -2232,9 +2182,10 @@ make_constant_oop (tree_node constExpr)
 	  elementOOP = make_constant_oop (arrayElt->v_list.value);
 	  result = OOP_TO_OBJ (resultOOP);
 	  result->data[i] = elementOOP;
+	  if (IS_OOP (elementOOP))
+	    MAKE_OOP_READONLY (elementOOP, true);
 	}
 
-      MAKE_OOP_READONLY (resultOOP, true);
       INC_RESTORE_POINTER (incPtr);
       return (resultOOP);
     }
@@ -2297,11 +2248,11 @@ compute_selector (tree_node selectorExpr)
       || selectorExpr->nodeType == TREE_BINARY_EXPR)
     return (selectorExpr->v_expr.selector);
   else
-    return (compute_keyword_selector (selectorExpr->v_expr.expression));
+    return (_gst_compute_keyword_selector (selectorExpr->v_expr.expression));
 }
 
 OOP
-compute_keyword_selector (tree_node keywordList)
+_gst_compute_keyword_selector (tree_node keywordList)
 {
   tree_node keyword;
   int len;
@@ -2341,9 +2292,9 @@ _gst_make_attribute (tree_node attribute_keywords)
   if (_gst_had_error)
     return _gst_nil_oop;
 
-  selectorOOP = compute_keyword_selector (attribute_keywords);
+  selectorOOP = _gst_compute_keyword_selector (attribute_keywords);
   numArgs = list_length (attribute_keywords);
-  argsArray = instantiate_with (_gst_array_class, numArgs, &argsArrayOOP);
+  argsArray = new_instance_with (_gst_array_class, numArgs, &argsArrayOOP);
   INC_ADD_OOP (argsArrayOOP);
 
   for (i = 0, keyword = attribute_keywords; keyword != NULL;
@@ -2352,9 +2303,9 @@ _gst_make_attribute (tree_node attribute_keywords)
       tree_node value = keyword->v_list.value;
       if (value->nodeType != TREE_CONST_EXPR)
 	{
-          tree_node stmt = _gst_make_statement_list (&value->location, value);
-          OOP result = _gst_execute_statements (NULL, stmt, true);
-          value = _gst_make_oop_constant (&stmt->location, result);
+          OOP result = _gst_execute_statements (NULL, value, UNDECLARED_NONE,
+					        true);
+          value = _gst_make_oop_constant (&value->location, result);
           if (!result)
 	    {
 	      _gst_had_error = true;
@@ -2468,6 +2419,11 @@ process_attribute (OOP messageOOP)
 	  return (-1);
 	}
     }
+  else if (selectorOOP == _gst_category_symbol)
+    {
+      this_method_category = arguments->data[0];
+      return (0);
+    }
   else
     {
       method_attributes *new_attr = (method_attributes *)
@@ -2650,11 +2606,14 @@ _gst_make_new_method (int primitiveIndex,
 		      OOP literals,
 		      bc_vector bytecodes,
 		      OOP class,
-		      OOP selector)
+		      OOP selector,
+		      OOP defaultCategoryOOP,
+		      int64_t startPos,
+		      int64_t endPos)
 {
   method_header header;
   int newFlags;
-  OOP method;
+  OOP method, methodDesc, sourceCode, category;
   inc_ptr incPtr;
 
   maximumStackDepth += numArgs + numTemps;
@@ -2731,7 +2690,27 @@ _gst_make_new_method (int primitiveIndex,
   header.numTemps = numTemps;
   header.intMark = 1;
 
-  method = method_new (header, literals, bytecodes, class, selector);
+  if (this_method_category)
+    {
+      category = this_method_category;
+      this_method_category = NULL;
+    }
+  else
+    category = defaultCategoryOOP;
+
+  if (IS_NIL (class))
+    sourceCode = _gst_nil_oop;
+  else
+    {
+      sourceCode = file_segment_new (startPos, endPos);
+      INC_ADD_OOP (sourceCode);
+    }
+
+  methodDesc = method_info_new (class, selector, method_attrs,
+				sourceCode, category);
+  INC_ADD_OOP (methodDesc);
+
+  method = method_new (header, literals, bytecodes, class, methodDesc);
   INC_RESTORE_POINTER (incPtr);
   return (method);
 }
@@ -2741,11 +2720,11 @@ method_new (method_header header,
 	    OOP literals,
 	    bc_vector bytecodes,
 	    OOP class,
-	    OOP selector)
+	    OOP methodDesc)
 {
   int numByteCodes;
   gst_compiled_method method;
-  OOP methodOOP, methodDesc;
+  OOP methodOOP;
   gst_object lit;
   int i;
 
@@ -2754,8 +2733,6 @@ method_new (method_header header,
   else
     numByteCodes = 0;
 
-  methodDesc = method_info_new (class, selector, method_attrs);
-  INC_ADD_OOP (methodDesc);
   method_attrs = NULL;
 
   method = (gst_compiled_method) instantiate_with (_gst_compiled_method_class,
@@ -2839,16 +2816,13 @@ _gst_block_new (int numArgs,
 OOP
 method_info_new (OOP class,
 		 OOP selector,
-		 method_attributes *attrs)
+		 method_attributes *attrs,
+		 OOP sourceCode,
+		 OOP categoryOOP)
 {
   method_attributes *next;
   gst_method_info methodInfo;
-  OOP sourceCode, methodInfoOOP;
-  inc_ptr incPtr;
-
-  incPtr = INC_SAVE_POINTER ();
-  sourceCode = file_segment_new ();
-  INC_ADD_OOP (sourceCode);
+  OOP methodInfoOOP;
 
   methodInfo =
     (gst_method_info) new_instance_with (_gst_method_info_class,
@@ -2856,7 +2830,7 @@ method_info_new (OOP class,
 					 &methodInfoOOP);
 
   methodInfo->sourceCode = sourceCode;
-  methodInfo->category = _gst_this_category;
+  methodInfo->category = categoryOOP;
   methodInfo->class = class;
   methodInfo->selector = selector;
 
@@ -2869,16 +2843,14 @@ method_info_new (OOP class,
       attrs = next;
     }
 			     
-  INC_RESTORE_POINTER (incPtr);
   return (methodInfoOOP);
 }
 
 OOP
-file_segment_new (void)
+file_segment_new (int64_t startPos, int64_t endPos)
 {
   OOP fileName, fileSegmentOOP;
   gst_file_segment fileSegment;
-  int64_t startPos = _gst_get_method_start_pos ();
   inc_ptr incPtr;
 
   if (startPos == -1)
@@ -2893,18 +2865,9 @@ file_segment_new (void)
 
   fileSegment->fileName = fileName;
   fileSegment->startPos = from_c_int_64 (startPos);
-  fileSegment->length =
-    from_c_int_64 (_gst_get_cur_file_pos () - startPos - 1);
+  fileSegment->length = from_c_int_64 (endPos - startPos);
 
   assert (to_c_int_64 (fileSegment->length) >= 0);
   INC_RESTORE_POINTER (incPtr);
   return (fileSegmentOOP);
 }
-
-void _gst_restore_primitive_number (OOP methodOOP, int *map)
-{
-  gst_compiled_method method;
-
-  method = (gst_compiled_method) OOP_TO_OBJ (methodOOP);
-  method->header.primitiveIndex = map[method->header.primitiveIndex];
-}
