@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -82,9 +104,9 @@ typedef struct method_header
 {
 #ifdef WORDS_BIGENDIAN
 #if SIZEOF_OOP == 8
-  unsigned dummy:32;		/* unused */
+  unsigned :32;		/* unused */
 #endif
-  unsigned:1;			/* sign - must be 0 */
+  unsigned :1;			/* sign - must be 0 */
   unsigned headerFlag:MTH_FLAG_BITS;	/* prim _gst_self, etc.  */
   unsigned primitiveIndex:MTH_PRIM_BITS;	/* index of primitve,
 						   or 0 */
@@ -100,9 +122,9 @@ typedef struct method_header
   unsigned primitiveIndex:MTH_PRIM_BITS;	/* index of primitve,
 						   or 0 */
   unsigned headerFlag:MTH_FLAG_BITS;	/* prim _gst_self, etc.  */
-  unsigned:1;			/* sign - must be 0 */
+  unsigned :1;			/* sign - must be 0 */
 #if SIZEOF_OOP == 8
-  unsigned dummy:32;		/* unused */
+  unsigned :32;		/* unused */
 #endif
 #endif				/* WORDS_BIGENDIAN */
 }
@@ -152,29 +174,29 @@ typedef struct block_header
 {
 #ifdef WORDS_BIGENDIAN
 #if SIZEOF_OOP == 8
-  unsigned dummy:32;		/* unused */
+  unsigned :32;		/* unused */
 #endif
-  unsigned:1;			/* sign - must be 0 */
+  unsigned :1;			/* sign - must be 0 */
   unsigned numArgs:BLK_ARGS_BITS;	/* number of arguments we have */
   unsigned numTemps:BLK_TEMPS_BITS;	/* number of _gst_temporaries
 					   we have */
   unsigned depth:BLK_DEPTH_BITS;	/* number of stack slots needed 
 					 */
-  unsigned unused:BLK_UNUSED_BITS;
+  unsigned :BLK_UNUSED_BITS;
   unsigned clean:BLK_CLEAN_BITS;	/* behavior of block */
   unsigned intMark:1;		/* flag this as an Int */
 #else
   unsigned intMark:1;		/* flag this as an Int */
   unsigned clean:BLK_CLEAN_BITS;	/* behavior of block */
-  unsigned unused:BLK_UNUSED_BITS;
+  unsigned :BLK_UNUSED_BITS;
   unsigned depth:BLK_DEPTH_BITS;	/* number of stack slots needed 
 					 */
   unsigned numTemps:BLK_TEMPS_BITS;	/* number of _gst_temporaries
 					   we have */
   unsigned numArgs:BLK_ARGS_BITS;	/* number of arguments we have */
-  unsigned:1;			/* sign - must be 0 */
+  unsigned :1;			/* sign - must be 0 */
 #if SIZEOF_OOP == 8
-  unsigned dummy:32;		/* unused */
+  unsigned :32;		/* unused */
 #endif
 #endif
 }
@@ -219,8 +241,9 @@ extern OOP _gst_last_returned_value
   ATTRIBUTE_HIDDEN;
 
 /* This flag controls whether byte codes are printed after
-   compilation.  */
-extern mst_Boolean _gst_declare_tracing 
+   compilation.  If > 1, it applies also to code not invoked by
+   the user.  */
+extern int _gst_declare_tracing 
   ATTRIBUTE_HIDDEN;
 
 /* If true, the compilation of a set of methods will be skipped
@@ -241,13 +264,19 @@ extern mst_Boolean _gst_had_error
 extern mst_Boolean _gst_untrusted_methods 
   ATTRIBUTE_HIDDEN;
 
-/* Called to compile and execute an "immediate expression"; i.e. a set
-   of Smalltalk statements that are not part of a method definition.
-   The parse trees are in TEMPORARIES and STATEMENTS.  Return the object
-   that was returned by the expression.  */
-extern OOP _gst_execute_statements (tree_node temporaries,
-				    tree_node statements,
-				    mst_Boolean quiet) 
+/* Given a tree_node of type TREE_KEYWORD_LIST, pick out and concatenate
+   the keywords, turn them into a symbol OOP and return that symbol.  */
+extern OOP _gst_compute_keyword_selector (tree_node selectorExpr)
+  ATTRIBUTE_HIDDEN;
+
+/* Called to compile and execute an "immediate expression"; i.e. a Smalltalk
+   statement that is not part of a method definition and where temporaries are
+   declared automatically.  The parse trees are in TEMPS and STATEMENTS.
+   Return the object that was returned by the expression.  */
+extern OOP _gst_execute_statements (tree_node temps,
+				    tree_node statement,
+				    enum undeclared_strategy undeclared,
+				    mst_Boolean quiet)
   ATTRIBUTE_HIDDEN;
 
 /* This function will print a message describing the method category
@@ -313,13 +342,16 @@ extern OOP _gst_compile_method (tree_node method,
    LITERALS is a Smalltalk Array containing the literals, or nil if we
    retrieve it from the array internal to comp.c */
 extern OOP _gst_make_new_method (int primitiveIndex,
-			int numArgs,
-			int numTemps,
-			int maximumStackDepth,
-			OOP literals,
-			bc_vector bytecodes,
-			OOP class,
-			OOP selector) 
+				 int numArgs,
+				 int numTemps,
+				 int maximumStackDepth,
+				 OOP literals,
+				 bc_vector bytecodes,
+				 OOP class,
+				 OOP selector,
+				 OOP defaultCategoryOOP,
+				 int64_t startPos,
+				 int64_t endPos) 
   ATTRIBUTE_HIDDEN;
 
 /* This function looks for the UndefinedObject>>#__terminate method
@@ -353,17 +385,6 @@ extern int _gst_add_forced_object (OOP oop)
 extern OOP _gst_make_attribute (tree_node attribute_keywords) 
   ATTRIBUTE_HIDDEN;
 
-/* This routine is called during image loading to restore the
-   primitive number in a CompiledMethod.  This is because we achieve
-   better binary compatibility if we keep the primitives name
-   invariant, rather than the primitive index, across image loads.
-   This allows us to keep the primitive list in prims.inl in a logical
-   order rather than having to add at the end of the list.  MAP
-   associates primitive numbers in the saved image (obtained by the
-   saved VMPrimitives dictionary) to primitive numbers in this VM.  */
-extern void _gst_restore_primitive_number (OOP methodOOP, int *map) 
-  ATTRIBUTE_HIDDEN;
-
 /* Process the attributes in ARRAYOOP, return the primitive number
    (so far, this is the only attribute we honor), or -1 for a bad
    primitive number.  */
