@@ -318,6 +318,7 @@ extern OOP _gst_compiled_block_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_compiled_code_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_compiled_method_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_context_part_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_continuation_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_date_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_delay_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_dictionary_class ATTRIBUTE_HIDDEN;
@@ -362,7 +363,6 @@ extern OOP _gst_permission_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_positionable_stream_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_process_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_processor_scheduler_class ATTRIBUTE_HIDDEN;
-extern OOP _gst_random_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_read_stream_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_read_write_stream_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_root_namespace_class ATTRIBUTE_HIDDEN;
@@ -380,12 +380,18 @@ extern OOP _gst_sym_link_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_symbol_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_system_dictionary_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_time_class ATTRIBUTE_HIDDEN;
-extern OOP _gst_token_stream_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_true_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_undefined_object_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_unicode_character_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_unicode_string_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_variable_binding_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_array_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_set_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_key_dictionary_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_value_lookup_table_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_identity_set_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_key_identity_dictionary_class ATTRIBUTE_HIDDEN;
+extern OOP _gst_weak_value_identity_dictionary_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_write_stream_class ATTRIBUTE_HIDDEN;
 extern OOP _gst_processor_oop ATTRIBUTE_HIDDEN;
 
@@ -418,9 +424,16 @@ extern OOP _gst_class_variable_dictionary (OOP class_oop)
   ATTRIBUTE_HIDDEN;
 
 /* This finds the key SYMBOL into the dictionary POOLOOP and, if any,
-   in all of its super-namespaces.  */
-extern OOP _gst_find_shared_pool_variable (OOP poolOOP,
-					   OOP symbol)
+   in all of its super-namespaces.  Returns the association.  */
+extern OOP _gst_namespace_association_at (OOP poolOOP,
+					  OOP symbol)
+  ATTRIBUTE_PURE 
+  ATTRIBUTE_HIDDEN;
+
+/* This finds the key SYMBOL into the dictionary POOLOOP and, if any,
+   in all of its super-namespaces.  Returns the value.  */
+extern OOP _gst_namespace_at (OOP poolOOP,
+			      OOP symbol)
   ATTRIBUTE_PURE 
   ATTRIBUTE_HIDDEN;
 
@@ -466,13 +479,6 @@ extern OOP _gst_message_new_args (OOP selectorOOP,
 				  OOP argsArray) 
   ATTRIBUTE_HIDDEN;
 
-/* Create a new DirectedMessage object for the given RECEIVER,
-   SELECTOR (a Symbol) and Array of arguments.  */
-extern OOP _gst_directed_message_new_args (OOP receiverOOP,
-					   OOP selectorOOP,
-					   OOP argsArray) 
-  ATTRIBUTE_HIDDEN;
-
 /* Returns the name of CLASS_OOP (a Smalltalk Class) */
 extern OOP _gst_get_class_symbol (OOP class_oop)
   ATTRIBUTE_PURE 
@@ -610,10 +616,10 @@ extern void _gst_set_oop_bytes (OOP byteArrayOOP,
 extern void _gst_free_cobject (OOP cObjOOP) 
   ATTRIBUTE_HIDDEN;
 
-/* Loads the contents of the global variable from the Smalltalk
-   dictionary after NUMOOPS objects have been loaded from an
-   image.  */
-extern mst_Boolean _gst_init_dictionary_on_image_load (size_t numOOPs) 
+/* Loads the contents of the global variables from the Smalltalk dictionary
+   after an image has been restored.  PRIM_TABLE_MATCHES if true if the
+   table of primitives is already set up correctly.  */
+extern mst_Boolean _gst_init_dictionary_on_image_load (mst_Boolean prim_table_matches) 
   ATTRIBUTE_HIDDEN;
 
 /* Transforms a primitive name into a primitive index, looking up
