@@ -97,6 +97,7 @@ typedef struct YYLTYPE
 {
   int first_line;
   int first_column;
+  int64_t file_offset;
 } YYLTYPE;
 
 enum parser_state {
@@ -105,10 +106,16 @@ enum parser_state {
   PARSE_DOIT
 };
 
-typedef struct gst_parser {
+typedef struct gst_lookahead {
   int token;
   YYSTYPE val;
   YYLTYPE loc;
+} gst_lookahead;
+
+typedef struct gst_parser {
+  gst_lookahead la [4];
+  int la_size;
+  int la_first;
   enum parser_state state;
   jmp_buf recover;
 } gst_parser;
@@ -124,4 +131,7 @@ extern void _gst_parse_method (void)
 extern void _gst_parse_chunks (void) 
   ATTRIBUTE_HIDDEN;
 
+extern void _gst_print_tokens (gst_parser *p)
+  ATTRIBUTE_HIDDEN;
+
 #endif
