@@ -55,6 +55,57 @@
 #define GST_GSTPRIV_H
 
 #include "config.h"
+
+#include <sys/types.h>
+#include <stdio.h>
+#include <stddef.h>
+#include <setjmp.h>
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+#include <obstack.h>
+#include <fcntl.h>
+#include <stdarg.h>
+#include <math.h>
+#include <float.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <limits.h>
+#include <poll.h>
+#include <ctype.h>
+#include <wchar.h>
+#include <dirent.h>
+#include <sys/time.h>
+#include <time.h>
+
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
+
+#ifdef HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif
+
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #include "gst.h"
 
 /* Convenience macros to test the versions of GCC.  Note - they won't
@@ -256,35 +307,36 @@ enum {
 
   /* The grouping of all the flags which are not valid across image
      saves and loads.  */
-  F_RUNTIME = 0xFFF000U,
-
-  /* Set if the OOP is currently unused.  */
-  F_FREE = 0x10U,
+  F_RUNTIME = 0xFF8000U,
 
   /* Set if the references to the instance variables of the object
      are weak.  */
-  F_WEAK = 0x20U,
+  F_WEAK = 0x10U,
 
   /* Set if the object is read-only.  */
-  F_READONLY = 0x40U,
+  F_READONLY = 0x20U,
 
   /* Set if the object is a context and hence its instance variables
      are only valid up to the stack pointer.  */
-  F_CONTEXT = 0x80U,
+  F_CONTEXT = 0x40U,
 
   /* Answer whether we want to mark the key based on references found
      outside the object.  */
-  F_EPHEMERON = 0x100U,
+  F_EPHEMERON = 0x80U,
 
   /* Set for objects that live in oldspace.  */
-  F_OLD = 0x200U,
+  F_OLD = 0x100U,
 
   /* Set together with F_OLD for objects that live in fixedspace.  */
-  F_FIXED = 0x400U,
+  F_FIXED = 0x200U,
 
   /* Set for untrusted classes, instances of untrusted classes,
      and contexts whose receiver is untrusted.  */
-  F_UNTRUSTED = 0x800U,
+  F_UNTRUSTED = 0x400U,
+
+  /* Set for objects that were loaded from the image.  We never
+     garbage collect their contents, only the OOPs.  */
+  F_LOADED = 0x800U,
 
   /* Set to the number of bytes unused in an object with byte-sized
      instance variables.  Note that this field and the following one
@@ -539,18 +591,23 @@ extern OOP _gst_nil_oop
    because they are likely to be optimized.  */
 
 #if SIZEOF_OOP == 4
-# if defined(WORDS_BIGENDIAN) || !defined (HAVE_INET_SOCKETS)
+# if !defined(WORDS_BIGENDIAN) && defined (HAVE_INET_SOCKETS)
+#  define BYTE_INVERT(x) htonl((x))
+# elif defined _OS_OSBYTEORDERPPC_H
+#  define BYTE_INVERT(x) OSReadSwapInt32(&(x), 0)
+# else
 #  define BYTE_INVERT(x) \
         ((uintptr_t)((((uintptr_t)(x) & 0x000000ffU) << 24) | \
                      (((uintptr_t)(x) & 0x0000ff00U) <<  8) | \
                      (((uintptr_t)(x) & 0x00ff0000U) >>  8) | \
                      (((uintptr_t)(x) & 0xff000000U) >> 24)))
-# else
-#  define BYTE_INVERT(x) htonl((x))
 # endif
 
 #else /* SIZEOF_OOP == 8 */
-# define BYTE_INVERT(x) \
+# if defined _OS_OSBYTEORDERPPC_H
+#  define BYTE_INVERT(x) OSReadSwapInt64(&(x), 0)
+# else
+#  define BYTE_INVERT(x) \
         ((uintptr_t)((((uintptr_t)(x) & 0x00000000000000ffU) << 56) | \
                      (((uintptr_t)(x) & 0x000000000000ff00U) << 40) | \
                      (((uintptr_t)(x) & 0x0000000000ff0000U) << 24) | \
@@ -559,6 +616,7 @@ extern OOP _gst_nil_oop
                      (((uintptr_t)(x) & 0x0000ff0000000000U) >> 24) | \
                      (((uintptr_t)(x) & 0x00ff000000000000U) >> 40) | \
                      (((uintptr_t)(x) & 0xff00000000000000U) >> 56)))
+# endif
 #endif /* SIZEOF_OOP == 8 */
 
 /* The standard min/max macros...  */
@@ -573,56 +631,6 @@ extern OOP _gst_nil_oop
 #define MIN(x, y) 		( ((x) > (y)) ? (y) : (x) )
 #endif
 
-#include <sys/types.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <assert.h>
-#include <stdlib.h>
-#include <string.h>
-#include <obstack.h>
-#include <fcntl.h>
-#include <stdarg.h>
-#include <math.h>
-#include <float.h>
-#include <errno.h>
-#include <sys/stat.h>
-#include <limits.h>
-#include <poll.h>
-#include <ctype.h>
-#include <wchar.h>
-#include <dirent.h>
-#include <sys/time.h>
-#include <time.h>
-
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
-
-#ifdef HAVE_EXECINFO_H
-#include <execinfo.h>
-#endif
-
-#ifdef HAVE_SYS_FILE_H
-#include <sys/file.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_SYS_MMAN_H
-#include <sys/mman.h>
-#endif
-
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
-#ifdef HAVE_INTTYPES_H
-#include <inttypes.h>
-#endif
-
 #include "ansidecl.h"
 #include "signalx.h"
 #include "mathl.h"
@@ -647,13 +655,13 @@ extern OOP _gst_nil_oop
 #include "lib.h"
 #include "oop.h"
 #include "byte.h"
+#include "sym.h"
 #include "comp.h"
 #include "interp.h"
 #include "opt.h"
 #include "save.h"
 #include "str.h"
 #include "sysdep.h"
-#include "sym.h"
 #include "xlat.h"
 #include "mpz.h"
 #include "print.h"
