@@ -140,7 +140,7 @@ interp_jmp_buf;
 /* If this is true, for each byte code that is executed, we print on
    stdout the byte index within the current gst_compiled_method and a
    decoded interpretation of the byte code.  */
-mst_Boolean _gst_execution_tracing = false;
+int _gst_execution_tracing = 0;
 
 /* When this is true, and an interrupt occurs (such as SIGABRT),
    Smalltalk will terminate itself by making a core dump (normally it
@@ -152,7 +152,8 @@ mst_Boolean _gst_make_core_file = false;
 mst_Boolean _gst_non_interactive = true;
 
 /* The table of functions that implement the primitives.  */
-static prim_table_entry _gst_primitive_table[NUM_PRIMITIVES];
+prim_table_entry _gst_primitive_table[NUM_PRIMITIVES];
+prim_table_entry _gst_default_primitive_table[NUM_PRIMITIVES];
 
 /* Some performance counters from the interpreter: these
    count the number of special returns.  */
@@ -276,7 +277,7 @@ static interp_jmp_buf *reentrancy_jmp_buf = NULL;
 
 /* when this flag is on and execution tracing is in effect, the top of
    the stack is printed as well as the byte code */
-static mst_Boolean verbose_exec_tracing = false;
+static int verbose_exec_tracing = false;
 
 /* This is the bridge to the primitive operations in the GNU Smalltalk
    system.  This function invokes the proper primitive_func with the
@@ -473,7 +474,7 @@ static void unwind_context (void);
 /* Used to help minimize the number of primitives used to control the
    various debugging flags, this routine maps the variable's INDEX to the
    address of a boolean debug flag, which it returns.  */
-static inline mst_Boolean *bool_addr_index (int index) ATTRIBUTE_PURE;
+static inline int *bool_addr_index (int index) ATTRIBUTE_PURE;
 
 /* Check whether it is true that sending SENDSELECTOR to RECEIVER
    accepts NUMARGS arguments.  Note that the RECEIVER is only used to
@@ -1960,8 +1961,6 @@ semaphore_new (int signals)
   return (semaphoreOOP);
 }
 
-/* runs before every evaluation (_gst_execute_statements) and before GC turned on.
-   Note that we don't use the incubator because _gst_processor_oop is a global.  */
 void
 _gst_init_process_system (void)
 {
@@ -2026,7 +2025,7 @@ create_callin_process (OOP contextOOP)
   return (initialProcessOOP);
 }
 
-mst_Boolean *
+int *
 bool_addr_index (int index)
 {
   switch (index)
@@ -2127,25 +2126,6 @@ _gst_nvmsg_send (OOP receiver,
   currentProcessOOP = get_active_process ();
   change_process_context (processOOP);
 
-#if 0
-  if (_gst_kernel_initialized)
-    {
-      gst_array argsArray;
-      OOP argsArrayOOP;
-      argsArray = new_instance_with (_gst_array_class, numArgs, &argsArrayOOP);
-      for (i = 0; i < sendArgs; i++)
-        argsArray->data[i] = args[i];
-
-      INC_ADD_OOP (argsArrayOOP);
-      dirMessageOOP = _gst_directed_message_new_args (receiver, sendSelector, argsArrayOOP);
-
-      receiver = _gst_process_class;
-      sendSelector = _gst_start_execution_symbol;
-      sendArgs = 1;
-      args = &dirMessageOOP;
-    }
-#endif
-
   PUSH_OOP (receiver);
   for (i = 0; i < sendArgs; i++)
     PUSH_OOP (args[i]);
@@ -2462,7 +2442,7 @@ interrupt_handler (int sig)
 #ifdef HAVE_EXECINFO_H
       /* Don't print a backtrace, for example, if exiting during a
 	 compilation.  */
-      if ((_gst_verbosity > 2 && (ip || _gst_gc_running))
+      if ((_gst_verbosity == 3 && (ip || _gst_gc_running))
 	  || is_serious_error || sig == SIGUSR1)
 	{
           PTR array[11];
@@ -2620,7 +2600,16 @@ execute_primitive_operation (int primitive, volatile int numArgs)
 prim_table_entry *
 _gst_get_primitive_attributes (int primitive)
 {
-  return &_gst_primitive_table[primitive];
+  return &_gst_default_primitive_table[primitive];
+}
+
+void
+_gst_set_primitive_attributes (int primitive, prim_table_entry *pte)
+{
+  if (pte)
+    _gst_primitive_table[primitive] = *pte;
+  else
+    _gst_primitive_table[primitive] = _gst_default_primitive_table[0];
 }
 
 void
