@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -81,8 +81,7 @@ enum
   PRIM_IS_PIPE,                 /* no args */
   PRIM_MK_TEMP,                 /* base: */
   PRIM_GET_CHARS_AT,            /* data:from:to:absOfs: */
-  PRIM_PUT_CHARS_AT,            /* data:from:to:absOfs: */
-  PRIM_SHUTDOWN_WRITE           /* shutdown */
+  PRIM_PUT_CHARS_AT             /* data:from:to:absOfs: */
 };
 
 /* These macros are used to quickly compute the number of words needed
@@ -254,8 +253,9 @@ extern unsigned long _gst_sample_counter
 
 /* If this is true, for each byte code that is executed, we print on
    stdout the byte index within the current gst_compiled_method and a
-   decoded interpretation of the byte code.  */
-extern mst_Boolean _gst_execution_tracing 
+   decoded interpretation of the byte code.  If > 1, it applies also
+   to code not invoked by the user.  */
+extern int _gst_execution_tracing 
   ATTRIBUTE_HIDDEN;
 
 /* When this is true, and an interrupt occurs (such as SIGSEGV),
@@ -427,9 +427,9 @@ extern void _gst_fixup_object_pointers (void)
 extern void _gst_restore_object_pointers (void) 
   ATTRIBUTE_HIDDEN;
 
-/* This runs before every evaluation (_gst_execute_statements) and
-   before GC turned on.  It creates an initial process if no process
-   is ready to run or if no process has been created yet.  */
+/* This runs before every evaluation and before GC turned on.  It creates an
+   initial process if no process is ready to run or if no process has been
+   created yet.  */
 extern void _gst_init_process_system (void) 
   ATTRIBUTE_HIDDEN;
 
@@ -543,12 +543,28 @@ prim_table_entry;
 #define PRIM_RETURN_SMALL_INTEGER	0x0100	/* 31 or 63 bits */
 #define PRIM_RETURN_SMALL_SMALLINTEGER	0x0300	/* 30 or 62 bits */
 
+/* The checksum of the table of primitive numbers.  Right now it is an MD5,
+   computed from part of the C source code of prims.inl.  We compare it when
+   loading an image, to avoid having to reload the primitive table.  */
+extern int _gst_primitives_md5[4];
+
+/* The table of functions that implement the primitives.  */
+extern prim_table_entry _gst_primitive_table[NUM_PRIMITIVES];
+extern prim_table_entry _gst_default_primitive_table[NUM_PRIMITIVES];
+ 
 /* This can be used to obtain information on a particular primitive
    operations in the GNU Smalltalk system.  */
 extern prim_table_entry * _gst_get_primitive_attributes (int primitive)
   ATTRIBUTE_PURE 
   ATTRIBUTE_HIDDEN;
 
+/* Dually, this maps the primitive number that will be used for running
+   the image, to the entry which was returned by _gst_get_primitive_attributes.
+   If PTE is NULL, the primitive will be invalid.  */
+extern void _gst_set_primitive_attributes (int primitive,
+					   prim_table_entry *pte)
+  ATTRIBUTE_HIDDEN;
+
 /* Initialize the table of primitives.  */
 extern void _gst_init_primitives () 
   ATTRIBUTE_HIDDEN;
