@@ -398,6 +398,43 @@ _gst_mpz_tdiv_qr (gst_mpz *quot, gst_mpz *rem, const gst_mpz *num, const gst_mpz
   alloca (0);
 }
 
+static void
+_gst_mpz_tdiv_q_ui (gst_mpz *quot, const gst_mpz *num, mp_limb_t den)
+{
+  mp_ptr np;
+  mp_ptr qp;
+  mp_size_t nsize = num->size;
+  mp_size_t sign_quotient = nsize;
+
+  nsize = ABS (nsize);
+
+  if (nsize == 0)
+    {
+      quot->size = 0;
+      return;
+    }
+
+  if (quot->alloc < nsize)
+    gst_mpz_realloc (quot, nsize);
+
+  qp = quot->d;
+  np = num->d;
+
+  /* Copy numerator to temporary space if it overlaps with the quotient.  */
+  if (np == qp)
+    {
+      mp_ptr tp;
+      tp = (mp_ptr) alloca (nsize * SIZEOF_MP_LIMB_T);
+      MPN_COPY (tp, np, nsize);
+      np = tp;
+    }
+
+  mpn_divrem_1 (qp, 0L, np, nsize, den);
+  nsize -=  qp[nsize - 1] == 0;
+  quot->size = sign_quotient >= 0 ? nsize : -nsize;
+  alloca (0);
+}
+
 mp_limb_t
 _gst_mpz_tdiv_qr_si (gst_mpz *quot, const gst_mpz *num, intptr_t den)
 {
@@ -1473,6 +1510,78 @@ _gst_mpz_xor (gst_mpz *res, const gst_mpz *op1, const gst_mpz *op2)
     res->size = -res_size;
   }
 }
+
+void
+_gst_mpz_divexact (gst_mpz *quot, const gst_mpz *num, const gst_mpz *den)
+{
+  mp_ptr qp, tp;
+  mp_srcptr np, dp;
+  mp_size_t nsize, dsize, qsize, d_zero_limbs;
+  int d_zero_bits;
+
+  nsize = ABS (num->size);
+  dsize = ABS (den->size);
+
+  np = num->d;
+  dp = den->d;
+
+  if (nsize == 0)
+    {
+      quot->size = 0;
+      return;
+    }
+
+  /* Check if newer GMP makes mpn_divexact_1 public.  */
+  if (dsize == 1)
+    {
+      _gst_mpz_tdiv_q_ui (quot, num, dp[0]);
+      if (den->size < 0)
+	quot->size = -quot->size;
+      return;
+    }
+
+  /* Avoid quadratic behaviour, but do it conservatively.  */
+  if (nsize - dsize > 1500)
+    {
+      gst_mpz r;
+      _gst_mpz_tdiv_qr (quot, &r, num, den);
+      _gst_mpz_clear (&r);
+      return;
+    }
+
+  /*  QUOT <-- NUM/2^r, T <-- DEN/2^r where = r number of twos in DEN.  */
+  d_zero_bits = mpn_scan1 (dp, 0);
+  d_zero_limbs = d_zero_bits / BITS_PER_MP_LIMB;
+  d_zero_bits &= BITS_PER_MP_LIMB - 1;
+  dp += d_zero_limbs;
+  dsize -= d_zero_limbs;
+  np += d_zero_limbs;
+  nsize -= d_zero_limbs;
+
+  /* Allocate where we place the result.  It must be nsize limbs big
+     because it also acts as a temporary area.  */
+  if (quot->alloc < nsize)
+    gst_mpz_realloc (quot, nsize);
+  qp = quot->d;
+
+  if (d_zero_bits != 0)
+    {
+      tp = (mp_ptr) alloca (dsize * SIZEOF_MP_LIMB_T);
+      mpn_rshift (tp, dp, dsize, d_zero_bits);
+      mpn_rshift (qp, np, nsize, d_zero_bits);
+
+      dsize -= tp[dsize - 1] == 0;
+      nsize -= qp[nsize - 1] == 0;
+      dp = tp;
+    }
+  else
+    MPN_COPY(qp, np, nsize);
+
+  qsize = nsize - dsize + 1;
+  mpn_bdivmod (qp, qp, nsize, dp, dsize, qsize * GMP_NUMB_BITS);
+  quot->size = (num->size ^ den->size) >= 0 ? qsize : -qsize;
+}
+
 void
 _gst_mpz_from_oop(gst_mpz *mpz, OOP srcOOP)
 {
