@@ -103,7 +103,7 @@ OOP _gst_true_oop = NULL;
 OOP _gst_false_oop = NULL;
 
 /* This is true to show a message whenever a GC happens.  */
-mst_Boolean _gst_gc_message = true;
+int _gst_gc_message = true;
 
 /* This is != 0 in the middle of a GC.  */
 int _gst_gc_running = 0;
@@ -426,13 +426,13 @@ void _gst_update_object_memory_oop (OOP oop)
 }
 
 void
-_gst_init_oop_table (size_t size)
+_gst_init_oop_table (PTR address, size_t size)
 {
   int i;
 
   oop_heap = NULL;
   for (i = MAX_OOP_TABLE_SIZE; i && !oop_heap; i >>= 1)
-    oop_heap = _gst_heap_create (i * sizeof (struct oop_s));
+    oop_heap = _gst_heap_create (address, i * sizeof (struct oop_s));
 
   if (!oop_heap)
     nomemory (true);
@@ -472,7 +472,6 @@ void
 alloc_oop_table (size_t size)
 {
   size_t bytes;
-  OOP oop;
 
   _gst_mem.ot_size = size;
   bytes = (size - FIRST_OOP_INDEX) * sizeof (struct oop_s);
@@ -487,16 +486,6 @@ alloc_oop_table (size_t size)
   _gst_false_oop = &_gst_mem.ot[FALSE_OOP_INDEX];
 
   _gst_mem.num_free_oops = size;
-
-  /* mark the OOPs as available */
-  PREFETCH_START (_gst_mem.ot, PREF_WRITE | PREF_NTA);
-  for (oop = _gst_mem.ot;
-       oop < &_gst_mem.ot[_gst_mem.ot_size]; oop++)
-    {
-      PREFETCH_LOOP (oop, PREF_WRITE | PREF_NTA);
-      oop->flags = F_FREE;
-    }
-
   _gst_mem.first_allocated_oop = _gst_mem.ot;
   _gst_mem.last_allocated_oop = _gst_mem.last_swept_oop = _gst_mem.ot - 1;
   _gst_mem.highest_swept_oop = _gst_mem.ot;
@@ -506,7 +495,6 @@ mst_Boolean
 _gst_realloc_oop_table (size_t newSize)
 {
   size_t bytes;
-  OOP oop;
 
   bytes = (newSize - _gst_mem.ot_size) * sizeof (struct oop_s);
   if (bytes < 0)
@@ -520,15 +508,6 @@ _gst_realloc_oop_table (size_t newSize)
       return (false);
     }
 
-  /* mark the new OOPs as available */
-  PREFETCH_START (&_gst_mem.ot[_gst_mem.ot_size], PREF_WRITE | PREF_NTA);
-  for (oop = &_gst_mem.ot[_gst_mem.ot_size];
-       oop < &_gst_mem.ot[newSize]; oop++)
-    {
-      PREFETCH_LOOP (oop, PREF_WRITE | PREF_NTA);
-      oop->flags = F_FREE;
-    }
-
   _gst_mem.num_free_oops += newSize - _gst_mem.ot_size;
   _gst_mem.ot_size = newSize;
   return (true);
@@ -747,18 +726,22 @@ _gst_make_oop_fixed (OOP oop)
   if (oop->flags & F_FIXED)
     return;
 
-  size = SIZE_TO_BYTES (TO_INT(oop->object->objSize));
-  newObj = (gst_object) _gst_mem_alloc (_gst_mem.fixed, size);
-  if (!newObj)
-    abort ();
+  if ((oop->flags & F_LOADED) == 0)
+    {
+      size = SIZE_TO_BYTES (TO_INT(oop->object->objSize));
+      newObj = (gst_object) _gst_mem_alloc (_gst_mem.fixed, size);
+      if (!newObj)
+        abort ();
 
-  memcpy (newObj, oop->object, size);
-  if (oop->flags & F_OLD)
-    _gst_mem_free (_gst_mem.old, oop->object);
-  else
-    _gst_mem.numOldOOPs++;
+      memcpy (newObj, oop->object, size);
+      if ((oop->flags & F_OLD) == 0)
+	_gst_mem.numOldOOPs++;
+      else
+        _gst_mem_free (_gst_mem.old, oop->object);
+
+      oop->object = newObj;
+    }
 
-  oop->object = newObj;
   oop->flags &= ~(F_SPACES | F_POOLED);
   oop->flags |= F_OLD | F_FIXED;
 }
@@ -778,7 +761,6 @@ _gst_tenure_oop (OOP oop)
         abort ();
 
       memcpy (newObj, oop->object, size);
-
       _gst_mem.numOldOOPs++;
 
       oop->object = newObj;
@@ -1051,7 +1033,7 @@ _gst_compact (size_t new_heap_limit)
        oop < &_gst_mem.ot[_gst_mem.ot_size]; oop++)
     {
       PREFETCH_LOOP (oop, PREF_READ | PREF_NTA);
-      if ((oop->flags & (F_OLD | F_FIXED)) == F_OLD)
+      if ((oop->flags & (F_OLD | F_FIXED | F_LOADED)) == F_OLD)
         {
           gst_object new;
           size_t size = SIZE_TO_BYTES (TO_INT (oop->object->objSize));
@@ -1093,7 +1075,7 @@ _gst_global_gc (int next_allocation)
     
   if (!_gst_gc_running++
       && _gst_gc_message
-      && _gst_verbosity > 0
+      && _gst_verbosity >= 2
       && !_gst_regression_testing)
     {
       /* print the first part of this message before we finish
@@ -1163,7 +1145,7 @@ _gst_global_gc (int next_allocation)
 
   if (!--_gst_gc_running
       && _gst_gc_message
-      && _gst_verbosity > 0
+      && _gst_verbosity >= 2
       && !_gst_regression_testing)
     {
       fprintf (stderr, "%s\"\n", s);
@@ -1423,17 +1405,19 @@ _gst_sweep_oop (OOP oop)
       _gst_mem.numOldOOPs--;
       stats.reclaimedOldSpaceBytesSinceLastGlobalGC +=
 	SIZE_TO_BYTES (TO_INT (OOP_TO_OBJ (oop)->objSize));
-      _gst_mem_free (_gst_mem.fixed, oop->object);
+      if ((oop->flags & F_LOADED) == 0)
+        _gst_mem_free (_gst_mem.fixed, oop->object);
     }
   else if UNCOMMON (oop->flags & F_OLD)
     {
       _gst_mem.numOldOOPs--;
       stats.reclaimedOldSpaceBytesSinceLastGlobalGC +=
 	SIZE_TO_BYTES (TO_INT (OOP_TO_OBJ (oop)->objSize));
-      _gst_mem_free (_gst_mem.old, oop->object);
+      if ((oop->flags & F_LOADED) == 0)
+        _gst_mem_free (_gst_mem.old, oop->object);
     }
 
-  oop->flags = F_FREE;
+  oop->flags = 0;
 }
 
 void
@@ -1976,20 +1960,13 @@ _gst_copy_an_oop (OOP oop)
 	  printf ("Invalid size for OOP %p (%p)\n", oop, obj);
 	  abort ();
 	}
-#endif
 
-      if UNCOMMON (oop->flags & F_FREE)
+      if UNCOMMON (oop->flags == 0)
 	{
 	  printf ("Free OOP %p was referenced\n", oop);
-	  /* If we're lucky, we can print some information on it...
-	     if we're not, we segfault: not that bad given that
-	     we'll abort pretty soon! */
-	  oop->flags &= ~F_FREE;
-	  _gst_display_oop (oop);
 	  abort ();
 	}
 
-#if defined (GC_DEBUGGING)
       if UNCOMMON ((oop->flags & F_OLD) ||
 	  IS_SURVIVOR_ADDR(obj, _gst_mem.active_half == &_gst_mem.surv[1]))
         {
