@@ -220,7 +220,7 @@ struct memory_space
   /* The OOP flag corresponding to the active survivor space */
   int active_flag;
 
-  /* The OOP flag corresponding to the inactive survivor space, plus F_FREE */
+  /* The OOP flag corresponding to the inactive survivor space.  */
   int live_flags;
 
   /* These hold onto the object incubator's state */
@@ -257,7 +257,7 @@ struct memory_space
 };
 
 /* This is true to show a message whenever a GC happens.  */
-extern mst_Boolean _gst_gc_message 
+extern int _gst_gc_message 
   ATTRIBUTE_HIDDEN;
 
 /* This is true in the middle of a GC.  */
@@ -323,12 +323,12 @@ extern void _gst_init_mem (size_t eden, size_t survivor, size_t old,
            	           int space_grow_rate) 
   ATTRIBUTE_HIDDEN;
 
-/* Initialize an OOP table of SIZE bytes.  Initially, all the OOPs are
-   free list so that's just how we initialize them.  We do
-   as much initialization as we can, but we're called before classses
-   are defined, so things that have definite classes must wait until
-   the classes are defined.  */
-extern void _gst_init_oop_table (size_t size) 
+/* Initialize an OOP table of SIZE bytes, trying at the given address if
+   possible.  Initially, all the OOPs are on the free list so that's
+   just how we initialize them.  We do as much initialization as we can,
+   but we're called before classses are defined, so things that have
+   definite classes must wait until the classes are defined.  */
+extern void _gst_init_oop_table (PTR address, size_t size) 
   ATTRIBUTE_HIDDEN;
 
 /* Dump the entire contents of the OOP table.  Mainly for debugging
