@@ -320,7 +320,7 @@ _gst_classify_addr (void *addr)
 void
 _gst_display_oop_short (OOP oop)
 {
-  if (oop->flags & F_FREE)
+  if (IS_OOP_FREE (oop))
     printf ("%-10p   Free\n", oop);
   else
     {
@@ -329,9 +329,10 @@ _gst_display_oop_short (OOP oop)
             oop->flags & F_WEAK ? "Weak" :
             oop->flags & F_EPHEMERON ? "Ephemeron" : "",
 
-            oop->flags & _gst_mem.active_flag ? "To-space" :
             oop->flags & F_FIXED ? "Fixed" :
-            oop->flags & F_OLD ? "Old" : "From-space",
+            oop->flags & F_LOADED ? "Permanent" :
+            oop->flags & F_OLD ? "Old" :
+            oop->flags & _gst_mem.active_flag ? "To-space" : "From-space",
 
 	    IS_EDEN_ADDR (oop->object) ? "Eden" :
 	    IS_SURVIVOR_ADDR (oop->object, 0) ? "Surv (Even)" :
@@ -350,7 +351,7 @@ _gst_display_oop (OOP oop)
       return;
     }
 
-  if (oop->flags & F_FREE)
+  if (IS_OOP_FREE (oop))
     printf ("%-10p   Free\n", oop);
   else
     {
@@ -359,9 +360,10 @@ _gst_display_oop (OOP oop)
             oop->flags & F_WEAK ? "Weak" :
             oop->flags & F_EPHEMERON ? "Ephemeron" : "",
 
-            oop->flags & _gst_mem.active_flag ? "To-space" :
             oop->flags & F_FIXED ? "Fixed" :
-            oop->flags & F_OLD ? "Old" : "From-space",
+            oop->flags & F_LOADED ? "Permanent" :
+            oop->flags & F_OLD ? "Old" :
+            oop->flags & _gst_mem.active_flag ? "To-space" : "From-space",
 
 	    IS_EDEN_ADDR (oop->object) ? "Eden" :
 	    IS_SURVIVOR_ADDR (oop->object, 0) ? "Surv (Even)" :
