@@ -110,6 +110,9 @@ typedef struct save_file_header
   size_t big_object_threshold;
   size_t grow_threshold_percent;
   size_t space_grow_rate;
+  size_t num_free_oops;
+  intptr_t ot_base;
+  intptr_t prim_table_md5[16 / sizeof (intptr_t)]; /* checksum for the primitive table */
 }
 save_file_header;
 
@@ -130,23 +133,37 @@ static off_t file_size;
 static off_t file_pos;
 
 /* Whether we are using mmap to read the file.  */
-static mst_Boolean use_mmap;
+static mst_Boolean buf_used_mmap;
 
 
+/* This function establishes a buffer of size NUMBYTES for writes.  */
+static void buffer_write_init (int imageFd,
+			       int numBytes);
+
+/* This function flushes and frees the buffer used for writes.  */
+static void buffer_write_flush (int imageFd);
+
 /* This function buffers writes to the image file whose descriptor is
-   IMAGEFD.  It should be called with DATA == NULL and N > 0 to
-   establish the buffer size, with DATA == NULL and N == 0 to flush
-   the data and free the buffer, and the same as write(2) to do the
-   actual I/O.  */
+   IMAGEFD.  */
 static void buffer_write (int imageFd,
 			  PTR data,
 			  int numBytes);
 
+/* This function establishes a buffer of size NUMBYTES for reads.  */
+static void buffer_read_init (int imageFd,
+			      int numBytes);
+
+/* This function frees the buffer used for reads.  */
+static void buffer_read_free (int imageFd);
+
+/* This function, which only works if memory-mapped I/O is used, advances
+   the buffer pointer by NUMBYTES and returns the pointer to the previous
+   value of the buffer pointer.  */
+static inline PTR buffer_advance (int imageFd,
+				  int numBytes);
+
 /* This function buffers reads from the image file whose descriptor
-   is IMAGEFD.  It should be called with DATA == NULL and N > 0 to
-   establish the buffer size, with DATA == NULL and N == 0 to flush
-   the data and free the buffer, and the same as write(2) to do the
-   actual I/O.  Memory-mapped I/O is used is possible.  */
+   is IMAGEFD.  Memory-mapped I/O is used is possible.  */
 static void buffer_read (int imageFd,
 			 PTR data,
 			 int numBytes);
@@ -161,26 +178,17 @@ static void buffer_fill (int imageFd);
 static void save_object (int imageFd,
 			 OOP oop);
 
-/* This function converts NUMFIXED absolute addresses at OBJ->data,
-   which are instance variables of the object, into relative ones.  */
-static inline void fixup_object (gst_object obj,
-			  int numPointers);
-
-/* This function converts NUMFIXED relative addresses at OBJ->data,
-   which are instance variables of the object, into absolute ones.  */
-static inline void restore_object (gst_object object,
-			    int numPointers);
+/* This function copies NUMBYTES from SRC to DEST, converting the first
+   NUMPOINTERS absolute addresses into relative ones (these are the
+   instance variables of the object).  */
+static inline void fixup_object (OOP oop, gst_object dest, gst_object src,
+				 int numBytes);
 
 /* This function inverts the endianness of SIZE long-words, starting at
    BUF.  */
 static inline void fixup_byte_order (PTR buf,
 			      size_t size);
 
-/* This function converts NUMFIXED relative addresses, starting at
-   OBJ->data, back to absolute form.  */
-static inline void restore_pointer_slots (gst_object obj,
-				          int numPointers);
-
 /* This function loads an OOP table made of OLDSLOTSUSED slots from
    the image file stored in the file whose descriptor is IMAGEFD.
    The fixup gets handled by load_normal_oops */
@@ -197,15 +205,16 @@ static void load_oop_table (int imageFd);
    Object data is loaded from the IMAGEFD file descriptor.  */
 static void load_normal_oops (int imageFd);
 
-/* This function stores the header of the image file into the file
+/* This function stores the header, HEADERP, of the image file into the file
    whose descriptor is IMAGEFD.  */
-static void save_file_version (int imageFd);
+static void save_file_version (int imageFd,
+			       struct save_file_header *headerp);
 
 /* This function loads into HEADERP the header of the image file
    without checking its validity.
    This data is loaded from the IMAGEFD file descriptor.  */
-static void load_file_version (int imageFd,
-			       save_file_header * headerp);
+static mst_Boolean load_file_version (int imageFd,
+				      save_file_header * headerp);
 
 /* This function walks the OOP table and converts all the relative
    addresses for the instance variables to absolute ones.  */
@@ -218,7 +227,7 @@ static inline void restore_oop_pointer_slots (OOP oop);
 /* This function prepares the OOP table to be written to the image
    file.  This contains the object sizes instead of the pointers,
    since addresses will be recalculated upon load.  */
-static struct oop_s *make_oop_table_to_be_saved (void);
+static struct oop_s *make_oop_table_to_be_saved (struct save_file_header *hdr);
 
 /* This function walks the OOP table and saves the data
    onto the file whose descriptor is IMAGEFD.  */
@@ -240,15 +249,13 @@ static mst_Boolean wrong_endianness;
    system.  */
 static int num_used_oops = 0;
 
-/* Convert to a relative offset from start of OOP table.  The offset
-   is 0 mod pointer-size, so it still looks like a pointer to the
-   IS_INT test.  */
-#define OOP_RELATIVE(obj) \
-  ( (OOP)((intptr_t)(obj) - (intptr_t)_gst_mem.ot) )
+/* Delta from the object table address used in the saved image, and
+   the one we allocate now.  */
+static intptr_t ot_delta;
 
 /* Convert from relative offset to actual oop table address.  */
 #define OOP_ABSOLUTE(obj) \
-  ( (OOP)((intptr_t)(obj) + (intptr_t)_gst_mem.ot) )
+  ( (OOP)((intptr_t)(obj) + ot_delta) )
 
 
 struct oop_s *myOOPTable = NULL;
@@ -257,6 +264,7 @@ mst_Boolean
 _gst_save_to_file (const char *fileName)
 {
   int imageFd;
+  save_file_header header;
 
   imageFd = _gst_open_file (fileName, "w");
   if (imageFd < 0)
@@ -267,10 +275,11 @@ _gst_save_to_file (const char *fileName)
   _gst_global_gc (0);
   _gst_finish_incremental_gc ();
 
-  myOOPTable = make_oop_table_to_be_saved ();
+  memzero (&header, sizeof (header));
+  myOOPTable = make_oop_table_to_be_saved (&header);
 
-  buffer_write (imageFd, NULL, WRITE_BUFFER_SIZE);
-  save_file_version (imageFd);
+  buffer_write_init (imageFd, WRITE_BUFFER_SIZE);
+  save_file_version (imageFd, &header);
 
 #ifdef SNAPSHOT_TRACE
   printf ("After saving header: %lld\n",
@@ -294,7 +303,7 @@ _gst_save_to_file (const char *fileName)
 	  lseek (imageFd, 0, SEEK_CUR));
 #endif /* SNAPSHOT_TRACE */
 
-  buffer_write (imageFd, NULL, 0);
+  buffer_write_flush (imageFd);
   close (imageFd);
 
   _gst_invoke_hook ("finishedSnapshot");
@@ -304,7 +313,7 @@ _gst_save_to_file (const char *fileName)
 
 
 struct oop_s *
-make_oop_table_to_be_saved (void)
+make_oop_table_to_be_saved (struct save_file_header *header)
 {
   OOP oop;
   struct oop_s *myOOPTable;
@@ -331,9 +340,9 @@ make_oop_table_to_be_saved (void)
     {
       if (IS_OOP_VALID_GC (oop))
 	{
-	  int numPointers;
-          myOOPTable[i].flags = oop->flags;
-	  numPointers = NUM_OOPS (oop->object);
+	  int numPointers = NUM_OOPS (oop->object);
+
+          myOOPTable[i].flags = (oop->flags & ~F_RUNTIME) | F_OLD;
 
 	  /* Cache the number of indexed instance variables.  We prefer
 	     to do more work upon saving (done once) than upon loading
@@ -346,8 +355,12 @@ make_oop_table_to_be_saved (void)
 	  myOOPTable[i].object = (gst_object) TO_INT (oop->object->objSize);
 	}
       else
-	myOOPTable[i].flags = F_FREE;
+	{
+	  myOOPTable[i].flags = 0;
+	  header->num_free_oops++;
+	}
     }
+
   return (myOOPTable);
 }
 
@@ -366,8 +379,8 @@ void
 save_object (int imageFd,
 	     OOP oop)
 {
-  gst_object object;
-  int numPointers, numBytes;
+  gst_object object, saveObject;
+  int numBytes;
 
 #ifdef SNAPSHOT_TRACE
   printf (">Save ");
@@ -375,36 +388,45 @@ save_object (int imageFd,
 #endif
 
   object = OOP_TO_OBJ (oop);
-  numPointers = NUM_OOPS (object);
 
   if (IS_OOP_FREE (oop))
     abort ();
 
-  fixup_object (object, numPointers);
   numBytes = sizeof (OOP) * TO_INT (object->objSize);
-  buffer_write (imageFd, object, numBytes);
-  restore_object (object, numPointers);
+  if (numBytes < 262144)
+    {
+      saveObject = alloca (numBytes);
+      fixup_object (oop, saveObject, object, numBytes);
+      buffer_write (imageFd, saveObject, numBytes);
+    }
+  else
+    {
+      saveObject = malloc (numBytes);
+      fixup_object (oop, saveObject, object, numBytes);
+      buffer_write (imageFd, saveObject, numBytes);
+      free (saveObject);
+    }
 }
 
 void
-save_file_version (int imageFd)
+save_file_version (int imageFd, struct save_file_header *headerp)
 {
-  save_file_header header;
+  memcpy (headerp->dummy, EXECUTE, strlen (EXECUTE));
+  memcpy (headerp->signature, SIGNATURE, strlen (SIGNATURE));
+  headerp->flags = FLAGS_WRITTEN;
+  headerp->version = VERSION_REQUIRED;
+  headerp->oopTableSize = num_used_oops;
+  headerp->edenSpaceSize = _gst_mem.eden.totalSize;
+  headerp->survSpaceSize = _gst_mem.surv[0].totalSize;
+  headerp->oldSpaceSize = _gst_mem.old->heap_limit;
 
-  memzero (&header, sizeof (header));
-  memcpy (header.dummy, EXECUTE, strlen (EXECUTE));
-  memcpy (header.signature, SIGNATURE, strlen (SIGNATURE));
-  header.flags = FLAGS_WRITTEN;
-  header.version = VERSION_REQUIRED;
-  header.oopTableSize = num_used_oops;
-  header.edenSpaceSize = _gst_mem.eden.totalSize;
-  header.survSpaceSize = _gst_mem.surv[0].totalSize;
-  header.oldSpaceSize = _gst_mem.old->heap_limit;
+  headerp->big_object_threshold = _gst_mem.big_object_threshold;
+  headerp->grow_threshold_percent = _gst_mem.grow_threshold_percent;
+  headerp->space_grow_rate = _gst_mem.space_grow_rate;
+  headerp->ot_base = (intptr_t) _gst_mem.ot_base;
+  memcpy (&headerp->prim_table_md5, _gst_primitives_md5, sizeof (_gst_primitives_md5));
 
-  header.big_object_threshold = _gst_mem.big_object_threshold;
-  header.grow_threshold_percent = _gst_mem.grow_threshold_percent;
-  header.space_grow_rate = _gst_mem.space_grow_rate;
-  buffer_write (imageFd, &header, sizeof (save_file_header));
+  buffer_write (imageFd, headerp, sizeof (save_file_header));
 }
 
 
@@ -424,8 +446,6 @@ _gst_load_from_file (const char *fileName)
   imageFd = _gst_open_file (fileName, "r");
   loaded = (imageFd >= 0) && load_snapshot (imageFd);
 
-  buffer_read (imageFd, NULL, 0);
-
   close (imageFd);
   return (loaded);
 }
@@ -434,32 +454,11 @@ mst_Boolean
 load_snapshot (int imageFd)
 {
   save_file_header header;
+  int prim_table_matches;
 
-  buffer_read (imageFd, NULL, READ_BUFFER_SIZE);
-  load_file_version (imageFd, &header);
-  if (strcmp (header.signature, SIGNATURE))
-    return (false);
-
-  /* different sizeof(PTR) not supported */
-  if (FLAG_CHANGED (header.flags, SLOT_SIZE_FLAG))
-    return (false);
-
-  if UNCOMMON ((wrong_endianness =
-         FLAG_CHANGED (header.flags, ENDIANNESS_FLAG)))
-    {
-      header.oopTableSize = BYTE_INVERT (header.oopTableSize);
-      header.edenSpaceSize = BYTE_INVERT (header.edenSpaceSize);
-      header.survSpaceSize = BYTE_INVERT (header.survSpaceSize);
-      header.oldSpaceSize = BYTE_INVERT (header.oldSpaceSize);
-      header.big_object_threshold = BYTE_INVERT (header.big_object_threshold);
-      header.grow_threshold_percent = BYTE_INVERT (header.grow_threshold_percent);
-      header.space_grow_rate = BYTE_INVERT (header.space_grow_rate);
-      header.version = BYTE_INVERT (header.version);
-    }
-
-  /* check for version mismatch; if so this image file is invalid */
-  if (header.version > VERSION_REQUIRED)
-    return (false);
+  buffer_read_init (imageFd, READ_BUFFER_SIZE);
+  if (!load_file_version (imageFd, &header))
+    return false;
 
 #ifdef SNAPSHOT_TRACE
   printf ("After loading header: %lld\n", file_pos + buf_pos);
@@ -469,10 +468,13 @@ load_snapshot (int imageFd)
 		 header.oldSpaceSize, header.big_object_threshold,
 		 header.grow_threshold_percent, header.space_grow_rate);
 
-  _gst_init_oop_table (MAX (header.oopTableSize * 2,
-		       INITIAL_OOP_TABLE_SIZE));
+  _gst_init_oop_table ((PTR) header.ot_base,
+		       MAX (header.oopTableSize * 2, INITIAL_OOP_TABLE_SIZE));
 
+  ot_delta = (intptr_t) (_gst_mem.ot_base) - header.ot_base;
   num_used_oops = header.oopTableSize;
+  _gst_mem.num_free_oops = header.num_free_oops;
+
   load_oop_table (imageFd);
 
 #ifdef SNAPSHOT_TRACE
@@ -485,9 +487,12 @@ load_snapshot (int imageFd)
   printf ("After loading objects: %lld\n", file_pos + buf_pos);
 #endif /* SNAPSHOT_TRACE */
 
-  restore_all_pointer_slots ();
+  if (ot_delta)
+    restore_all_pointer_slots ();
 
-  if (_gst_init_dictionary_on_image_load (num_used_oops))
+  prim_table_matches = !memcmp (header.prim_table_md5, _gst_primitives_md5,
+				sizeof (_gst_primitives_md5));
+  if (_gst_init_dictionary_on_image_load (prim_table_matches))
     {
 #ifdef SNAPSHOT_TRACE
       _gst_dump_oop_table ();
@@ -498,33 +503,55 @@ load_snapshot (int imageFd)
   return (false);
 }
 
-void
+mst_Boolean
 load_file_version (int imageFd,
 		   save_file_header * headerp)
 {
   buffer_read (imageFd, headerp, sizeof (save_file_header));
+  if (strcmp (headerp->signature, SIGNATURE))
+    return (false);
+
+  /* different sizeof(PTR) not supported */
+  if (FLAG_CHANGED (headerp->flags, SLOT_SIZE_FLAG))
+    return (false);
+
+  if UNCOMMON ((wrong_endianness =
+         FLAG_CHANGED (headerp->flags, ENDIANNESS_FLAG)))
+    {
+      headerp->oopTableSize = BYTE_INVERT (headerp->oopTableSize);
+      headerp->edenSpaceSize = BYTE_INVERT (headerp->edenSpaceSize);
+      headerp->survSpaceSize = BYTE_INVERT (headerp->survSpaceSize);
+      headerp->oldSpaceSize = BYTE_INVERT (headerp->oldSpaceSize);
+      headerp->big_object_threshold = BYTE_INVERT (headerp->big_object_threshold);
+      headerp->grow_threshold_percent = BYTE_INVERT (headerp->grow_threshold_percent);
+      headerp->space_grow_rate = BYTE_INVERT (headerp->space_grow_rate);
+      headerp->version = BYTE_INVERT (headerp->version);
+      headerp->num_free_oops = BYTE_INVERT (headerp->num_free_oops);
+      headerp->ot_base = BYTE_INVERT (headerp->ot_base);
+      headerp->prim_table_md5[0] = BYTE_INVERT (headerp->prim_table_md5[0]);
+      headerp->prim_table_md5[1] = BYTE_INVERT (headerp->prim_table_md5[1]);
+#if SIZEOF_OOP == 4
+      headerp->prim_table_md5[2] = BYTE_INVERT (headerp->prim_table_md5[2]);
+      headerp->prim_table_md5[3] = BYTE_INVERT (headerp->prim_table_md5[3]);
+#endif
+    }
+
+  /* check for version mismatch; if so this image file is invalid */
+  if (headerp->version > VERSION_REQUIRED)
+    return (false);
+
+  return (true);
 }
 
 void
 load_oop_table (int imageFd)
 {
-  OOP oop;
-
-  /* load in the valid OOP slots from previous dump */
+  /* Load in the valid OOP slots from previous dump.  The others are already
+     initialized to free (0).  */
   buffer_read (imageFd, _gst_mem.ot, sizeof (struct oop_s) * num_used_oops);
   if UNCOMMON (wrong_endianness)
     fixup_byte_order (_gst_mem.ot,
 		      sizeof (struct oop_s) * num_used_oops / sizeof (PTR));
-
-  /* mark the remaining ones as available */
-  PREFETCH_START (&_gst_mem.ot[num_used_oops], PREF_WRITE | PREF_NTA);
-  for (oop = &_gst_mem.ot[num_used_oops];
-       oop < &_gst_mem.ot[_gst_mem.ot_size]; oop++)
-    {
-      PREFETCH_LOOP (oop, PREF_WRITE | PREF_NTA);
-      oop->flags = F_FREE;
-      oop++;
-    }
 }
 
 
@@ -533,24 +560,24 @@ load_normal_oops (int imageFd)
 {
   OOP oop;
   gst_object object;
+  int i;
+  mst_Boolean use_copy_on_write
+    = buf_used_mmap && ~wrong_endianness && ot_delta == 0;
 
-  /* Now walk the oop table.  Start fixing the byte order.  */
+  /* Now walk the oop table.  Load the data (or get the addresses from the
+     mmap-ed area) and fix the byte order.  */
 
-  _gst_mem.num_free_oops = _gst_mem.ot_size - num_used_oops;
   _gst_mem.last_allocated_oop = &_gst_mem.ot[num_used_oops - 1];
   PREFETCH_START (_gst_mem.ot, PREF_WRITE | PREF_NTA);
-  for (oop = _gst_mem.ot; oop < &_gst_mem.ot[num_used_oops];
-       oop++)
+  for (oop = _gst_mem.ot, i = num_used_oops; i--; oop++)
     {
       size_t size;
+      intptr_t flags;
 
       PREFETCH_LOOP (oop, PREF_WRITE | PREF_NTA);
-      if (oop->flags & F_FREE)
-	{
-	  _gst_mem.num_free_oops++;
-	  oop->flags = F_FREE;	/* just free */
-	  continue;
-	}
+      flags = oop->flags;
+      if (IS_OOP_FREE (oop))
+	continue;
 
       /* FIXME: a small amount of garbage is saved that is produced
          by mourning the ephemerons right before GC.  We should probably
@@ -561,47 +588,41 @@ load_normal_oops (int imageFd)
 	 to create new-space objects.  The solution is not however as neat
 	 as possible.  */
 
-      oop->flags &= ~(F_SPACES | F_POOLED);
-      oop->flags |= F_OLD;
-
       _gst_mem.numOldOOPs++;
-      if (oop->flags & F_FIXED)
-	_gst_mem.numFixedOOPs++;
+      size = sizeof (PTR) * (size_t) oop->object;
+      if (use_copy_on_write)
+	{
+	  oop->flags |= F_LOADED;
+	  object = buffer_advance (imageFd, size);
+	}
 
-      size = (size_t) oop->object;
-      object = (gst_object) _gst_mem_alloc (
-	(oop->flags & F_FIXED) ? _gst_mem.fixed : _gst_mem.old,
-        size * sizeof (PTR));
+      else
+	{
+	  if (flags & F_FIXED)
+	    {
+	      _gst_mem.numFixedOOPs++;
+              object = (gst_object) _gst_mem_alloc (_gst_mem.fixed, size);
+	    }
+          else
+            object = (gst_object) _gst_mem_alloc (_gst_mem.old, size);
 
-      buffer_read (imageFd, object, sizeof (PTR) * size);
-      if (!IS_INT (object->objSize) || TO_INT (object->objSize) != size)
-	abort ();
+          buffer_read (imageFd, object, size);
+          if UNCOMMON (wrong_endianness)
+	    fixup_byte_order (object, 
+			      (flags & F_BYTE)
+			      ? OBJ_HEADER_SIZE_WORDS
+			      : size / sizeof (PTR));
 
-      object->objClass = OOP_ABSOLUTE (object->objClass);
-
-      /* Remove flags that are invalid after an image has been loaded.  */
-      oop->flags &= ~F_RUNTIME;
+	  /* Would be nice, but causes us to touch every page and lose most
+	     of the startup-time benefits of copy-on-write.  So we only
+	     do it in the slow case, anyway.  */
+	  if (object->objSize != FROM_INT ((size_t) oop->object))
+	    abort ();
+        }
 
       oop->object = object;
-      if UNCOMMON (wrong_endianness)
-	{
-	  OOP size = (OOP) BYTE_INVERT ((intptr_t) oop->object->objSize);
-	  fixup_byte_order (object, 
-			    (oop->flags & F_BYTE)
-			      ? OBJ_HEADER_SIZE_WORDS : TO_INT (size));
-	}
-
-      if (oop->flags & F_WEAK)
+      if (flags & F_WEAK)
 	_gst_make_oop_weak (oop);
-
-      if (oop->flags & F_CONTEXT)
-	{
-	  /* this is another quirk; this is not the best place to do
-	     it. We have to reset the nativeIPs so that we can find
-	     restarted processes and recompile their methods.  */
-	  gst_method_context context = (gst_method_context) object;
-	  context->native_ip = DUMMY_NATIVE_IP;
-	}
     }
 
   /* NUM_OOPS requires access to the instance spec in the class
@@ -610,13 +631,17 @@ load_normal_oops (int imageFd)
      do another pass here and fix the byte objects using the now
      correct class objects.  */
   if UNCOMMON (wrong_endianness)
-    for (oop = _gst_mem.ot; oop < &_gst_mem.ot[num_used_oops];
-         oop++)
+    for (oop = _gst_mem.ot, i = num_used_oops; i--; oop++)
       if (oop->flags & F_BYTE)
 	{
+	  OOP classOOP;
 	  object = OOP_TO_OBJ (oop);
-	  fixup_byte_order (object->data, NUM_OOPS (object));
+          classOOP = OOP_ABSOLUTE (object->objClass);
+	  fixup_byte_order (object->data, CLASS_FIXED_FIELDS (classOOP));
 	}
+
+  if (!use_copy_on_write)
+    buffer_read_free (imageFd);
 }
 
 
@@ -624,45 +649,77 @@ load_normal_oops (int imageFd)
    loading and saving */
 
 void
-fixup_object (gst_object obj,
-	      int numPointers)
+fixup_object (OOP oop, gst_object dest, gst_object src, int numBytes)
 {
-  OOP instOOP, class_oop, *i;
+  OOP class_oop;
+  memcpy (dest, src, numBytes);
 
-  class_oop = obj->objClass;
-  obj->objClass = OOP_RELATIVE (class_oop);
+  /* Do the heavy work on the objects now rather than at load time, in order
+     to make the loading faster.  In general, we should do this as little as
+     possible, because it's pretty hard: the three cases below for Process,
+     Semaphore and CallinProcess for example are just there to terminate all
+     CallinProcess objects.  */
 
-  for (i = obj->data; numPointers; i++, numPointers--)
+  class_oop = src->objClass;
+
+  if (oop->flags & F_CONTEXT)
     {
-      instOOP = *i;
-      if (IS_OOP (instOOP))
-	*i = OOP_RELATIVE (instOOP);
+      /* this is another quirk; this is not the best place to do
+         it. We have to reset the nativeIPs so that we can find
+         restarted processes and recompile their methods.  */
+      gst_method_context context = (gst_method_context) dest;
+      context->native_ip = DUMMY_NATIVE_IP;
     }
-}
 
-void
-restore_object (gst_object object,
-		     int numPointers)
-{
-  object->objClass = OOP_ABSOLUTE (object->objClass);
-  restore_pointer_slots (object, numPointers);
-}
-
-void
-restore_pointer_slots (gst_object obj,
-		       int numPointers)
-{
-  OOP instOOP, *i;
-
-  i = obj->data;
-  while (numPointers--)
+  else if (class_oop == _gst_callin_process_class)
     {
-      instOOP = *i;
-      if (IS_OOP (instOOP))
-	*i = OOP_ABSOLUTE (instOOP);
-
-      i++;
+      gst_process process = (gst_process) dest;
+      process->suspendedContext = _gst_nil_oop;
+      process->nextLink = _gst_nil_oop;
+      process->myList = _gst_nil_oop;
     }
+
+  else if (class_oop == _gst_process_class)
+    {
+      /* Find the new next link.  */
+      gst_process destProcess = (gst_process) dest;
+      gst_process next = (gst_process) src;
+      while (OOP_CLASS (next->nextLink) == _gst_callin_process_class)
+	next = (gst_process) OOP_TO_OBJ (next->nextLink);
+
+      destProcess->nextLink = next->nextLink;
+    }
+
+  else if (class_oop == _gst_semaphore_class)
+    {
+      /* Find the new first and last link.  */
+      gst_semaphore destSem = (gst_semaphore) dest;
+      gst_semaphore srcSem = (gst_semaphore) src;
+      OOP linkOOP = srcSem->firstLink;
+
+      destSem->firstLink = _gst_nil_oop;
+      destSem->lastLink = _gst_nil_oop;
+      while (!IS_NIL (linkOOP))
+	{
+	  gst_process process = (gst_process) OOP_TO_OBJ (linkOOP);
+	  if (process->objClass != _gst_callin_process_class)
+	    {
+	      if (IS_NIL (destSem->firstLink))
+		destSem->firstLink = linkOOP;
+	      destSem->lastLink = linkOOP;
+	    }
+	  linkOOP = process->nextLink;
+	}
+    }
+
+  /* The other case is to reset CFunctionDescriptor objects, so that we'll
+     relink the external functions when we reload the image.  */
+  else if (class_oop == _gst_c_func_descriptor_class)
+    {
+      gst_cfunc_descriptor desc = (gst_cfunc_descriptor) dest;
+      desc->cFunction = _gst_nil_oop;
+    }
+
 }
 
 void
@@ -681,14 +738,19 @@ restore_oop_pointer_slots (OOP oop)
 {
   int numPointers;
   gst_object object;
+  OOP *i;
 
   object = OOP_TO_OBJ (oop);
+  object->objClass = OOP_ABSOLUTE (object->objClass);
+
   if UNCOMMON ((oop->flags & F_COUNT) == F_COUNT)
     numPointers = NUM_OOPS (object);
   else
     numPointers = oop->flags >> F_COUNT_SHIFT;
 
-  restore_pointer_slots (object, numPointers);
+  for (i = object->data; numPointers--; i++)
+    if (IS_OOP (*i))
+      *i = OOP_ABSOLUTE (*i);
 }
 
 void
@@ -700,29 +762,27 @@ fixup_byte_order (PTR buf,
     *p = BYTE_INVERT (*p);
 }
 
+void
+buffer_write_init (int imageFd, int numBytes)
+{
+  buf = xmalloc (numBytes);
+  buf_size = numBytes;
+  buf_pos = 0;
+}
+
+void
+buffer_write_flush (int imageFd)
+{
+  _gst_full_write (imageFd, buf, buf_pos);
+  xfree (buf);
+  buf_pos = 0;
+}
 
 void
 buffer_write (int imageFd,
 	      PTR data,
 	      int numBytes)
 {
-  if UNCOMMON (!data)
-    {
-      if (numBytes)
-	{
-	  buf = xmalloc (numBytes);
-	  buf_size = numBytes;
-	}
-      else
-	{
-	  _gst_full_write (imageFd, buf, buf_pos);
-	  xfree (buf);
-	}
-
-      buf_pos = 0;
-      return;
-    }
-
   if UNCOMMON (buf_pos + numBytes > buf_size)
     {
       _gst_full_write (imageFd, buf, buf_pos);
@@ -742,40 +802,51 @@ void
 buffer_fill (int imageFd)
 {
   buf_pos = 0;
-  if (use_mmap)
-    {
+  read (imageFd, buf, buf_size);
+}
+
+void
+buffer_read_init (int imageFd, int numBytes)
+{
+  struct stat st;
+  fstat (imageFd, &st);
+  file_size = st.st_size;
+  file_pos = 0;
+
 #ifndef WIN32
-      if (buf)
-	_gst_osmem_free (buf, buf_size);
+  buf = mmap (NULL, file_size, PROT_READ, MAP_PRIVATE, imageFd, 0);
 
-      buf = mmap (NULL, buf_size, PROT_READ, MAP_SHARED, imageFd, file_pos);
-
-      if (buf != (PTR) -1)
-	{
-#ifdef HAVE_MADVISE
-#ifdef MADV_WILLNEED
-	  madvise (buf, buf_size, MADV_WILLNEED);
-#endif
-
-	  /* Ahem... this madvise causes a kernel OOPS on my machine!  */
-#if 0
-#ifdef MADV_SEQUENTIAL
-	  madvise (buf, buf_size, MADV_SEQUENTIAL);
-#endif
-#endif
-
-#endif /* HAVE_MADVISE */
-	  return;
-	}
+  if (buf != (PTR) -1)
+    {
+      buf_size = file_size;
+      buf_used_mmap = true;
+      return;
+    }
 #endif /* !WIN32 */
 
-      /* First non-mmaped input operation.  Allocate the buffer.  */
-      buf = xmalloc (buf_size);
-      use_mmap = false;
-    }
+  /* Non-mmaped input.  */
+  buf_used_mmap = false;
+  buf_size = numBytes;
+  buf = xmalloc (buf_size);
+  buffer_fill (imageFd);
+}
 
-  /* Cannot mmap the file, use read(2).  */
-  read (imageFd, buf, buf_size);
+void
+buffer_read_free (int imageFd)
+{
+  if (buf_used_mmap)
+    _gst_osmem_free (buf, buf_size);
+  else
+    xfree (buf);
+}
+
+PTR
+buffer_advance (int imageFd,
+	        int numBytes)
+{
+  PTR current_pos = buf + buf_pos;
+  buf_pos += numBytes;
+  return current_pos;
 }
 
 void
@@ -785,37 +856,6 @@ buffer_read (int imageFd,
 {
   char *data = (char *) pdata;
 
-  if UNCOMMON (!data)
-    {
-      if (numBytes)
-	{
-	  struct stat st;
-	  fstat (imageFd, &st);
-	  file_size = st.st_size;
-	  file_pos = 0;
-	  buf_size = numBytes;
-	  use_mmap = true;
-	  buf = NULL;
-	  buffer_fill (imageFd);
-	}
-      else
-	{
-	  if (use_mmap)
-	    _gst_osmem_free (buf, buf_size);
-	  else
-	    xfree (buf);
-	}
-
-      return;
-    }
-
-  /* Avoid triggering a SIGBUS.  */
-  if UNCOMMON (numBytes > file_size - file_pos)
-    {
-      memzero (data + file_size - file_pos, numBytes - (file_size - file_pos));
-      numBytes = file_size - file_pos;
-    }
-
   if UNCOMMON (numBytes > buf_size - buf_pos)
     {
       memcpy (data, buf + buf_pos, buf_size - buf_pos);
