@@ -11,7 +11,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2003,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -95,6 +95,12 @@
 # include <windows.h>
 #endif
 
+/* Get declaration of _NSGetExecutablePath on MacOS X 10.2 or newer.  */
+#if HAVE_MACH_O_DYLD_H
+# define ENUM_DYLD_BOOL
+# include <mach-o/dyld.h>
+#endif
+
 #if defined MAP_ANONYMOUS && !defined MAP_ANON
 # define MAP_ANON MAP_ANONYMOUS
 #endif
@@ -710,7 +716,6 @@ _gst_set_file_interrupt (int fd,
 char *
 _gst_get_cur_dir_name (void)
 {
-#ifdef HAVE_GETCWD
   char *cwd;
   char *ret;
   unsigned path_max;
@@ -741,11 +746,6 @@ _gst_get_cur_dir_name (void)
   xfree (cwd);
   errno = save_errno;
   return (NULL);
-#else
-  char name[PATH_MAX + 2];
-  getwd (name);
-  return (xstrdup (name));
-#endif
 }
 
 
@@ -776,30 +776,169 @@ _gst_set_file_access_times (const char *name, long new_atime, long new_mtime)
 
 
 char *
-_gst_get_full_file_name (const char *fileName)
+_gst_get_full_file_name (const char *name)
 {
-  char *fullFileName;
-  static char *fullPath = NULL;
-
-  /* Absolute paths don't need to change */
-  if (fileName[0] == '/')
-    return (xstrdup (fileName));
-#if defined WIN32 && !defined __CYGWIN__
-  if (fileName[0] == '\\' || (fileName[0] != '\0' && fileName[1] == ':'))
-    return (xstrdup (fileName));
+  char *rpath, *dest;
+  const char *start, *end, *rpath_limit;
+  long int path_max;
+#ifdef HAVE_READLINK
+  int num_links = 0;
+  char *extra_buf = NULL;
 #endif
 
-  /* Only need to do this once, then cache the result */
-  if (fullPath == NULL)
-    fullPath = _gst_get_cur_dir_name ();
-
-#if !defined WIN32 || defined __CYGWIN__
-  asprintf (&fullFileName, "%s/%s", fullPath, fileName);
+#ifdef PATH_MAX
+  path_max = PATH_MAX;
 #else
-  asprintf (&fullFileName, "%s\\%s", fullPath, fileName);
+  path_max = pathconf (name, _PC_PATH_MAX);
+  if (path_max <= 0)
+    path_max = 1024;
 #endif
 
-  return (fullFileName);
+  rpath = malloc (path_max);
+  if (rpath == NULL)
+    return NULL;
+  rpath_limit = rpath + path_max;
+
+#if defined WIN32 && !defined __CYGWIN__
+  {
+    DWORD retval = GetFullPathNameA (name, path_max, rpath, NULL);
+    if (retval > path_max)
+      {
+	rpath = realloc (rpath, retval);
+        retval = GetFullPathNameA (name, retval, rpath, NULL);
+      }
+    if (retval == 0
+	|| stat (rpath, &st) == -1)
+      goto error;
+  }
+#else
+  if (name[0] != '/')
+    {
+      if (!getcwd (rpath, path_max))
+	goto error;
+
+      dest = strchr (rpath, '\0');
+    }
+  else
+    {
+      rpath[0] = '/';
+      dest = rpath + 1;
+    }
+
+  for (start = end = name; *start; start = end)
+    {
+      struct stat st;
+
+      /* Skip sequence of multiple path-separators.  */
+      while (*start == '/')
+	++start;
+
+      /* Find end of path component.  */
+      for (end = start; *end && *end != '/'; ++end)
+	/* Nothing.  */;
+
+      if (end - start == 0)
+	break;
+      else if (end - start == 1 && start[0] == '.')
+	/* nothing */;
+      else if (end - start == 2 && start[0] == '.' && start[1] == '.')
+	{
+	  /* Back up to previous component, ignore if at root already.  */
+	  if (dest > rpath + 1)
+	    while ((--dest)[-1] != '/');
+	}
+      else
+	{
+	  size_t new_size;
+
+	  if (dest[-1] != '/')
+	    *dest++ = '/';
+
+	  if (dest + (end - start) >= rpath_limit)
+	    {
+	      ptrdiff_t dest_offset = dest - rpath;
+	      char *new_rpath;
+
+	      new_size = rpath_limit - rpath;
+	      if (end - start + 1 > path_max)
+		new_size += end - start + 1;
+	      else
+		new_size += path_max;
+	      new_rpath = (char *) realloc (rpath, new_size);
+	      if (new_rpath == NULL)
+		goto error;
+	      rpath = new_rpath;
+	      rpath_limit = rpath + new_size;
+
+	      dest = rpath + dest_offset;
+	    }
+
+	  memcpy (dest, start, end - start);
+	  dest += end - start;
+	  *dest = '\0';
+
+	  if (lstat (rpath, &st) < 0)
+	    goto error;
+
+#if HAVE_READLINK
+	  if (S_ISLNK (st.st_mode))
+	    {
+	      char *buf;
+	      size_t len;
+	      int n;
+
+	      if (++num_links > MAXSYMLINKS)
+		{
+		  errno = ELOOP;
+		  goto error;
+		}
+
+	      buf = alloca (path_max);
+	      n = readlink (rpath, buf, path_max);
+	      if (n < 0)
+		{
+		  int saved_errno = errno;
+		  errno = saved_errno;
+		  goto error;
+		}
+	      buf[n] = '\0';
+	      if (!extra_buf)
+		extra_buf = alloca (path_max);
+
+	      len = strlen (end);
+	      if ((long int) (n + len) >= path_max)
+		{
+		  errno = ENAMETOOLONG;
+		  goto error;
+		}
+
+	      /* Careful here, end may be a pointer into extra_buf... */
+	      memmove (&extra_buf[n], end, len + 1);
+	      name = end = memcpy (extra_buf, buf, n);
+
+	      if (buf[0] == '/')
+		dest = rpath + 1;	/* It's an absolute symlink */
+	      else
+		/* Back up to previous component, ignore if at root already: */
+		if (dest > rpath + 1)
+		  while ((--dest)[-1] != '/');
+	    }
+#endif
+	}
+    }
+  if (dest > rpath + 1 && dest[-1] == '/')
+    --dest;
+  *dest = '\0';
+#endif
+  return rpath;
+
+error:
+  {
+    int saved_errno = errno;
+    free (rpath);
+    errno = saved_errno;
+  }
+  return NULL;
 }
 
 
@@ -834,6 +973,158 @@ _gst_file_is_executable (const char *fileName)
 {
   return (access (fileName, X_OK) == 0);
 }
+
+#ifdef __linux__
+/* File descriptor of the executable, used for double checking.  */
+static int executable_fd = -1;
+#endif
+
+/* Tests whether a given pathname may belong to the executable.  */
+static mst_Boolean
+maybe_executable (const char *filename)
+{
+  if (!_gst_file_is_executable (filename))
+    return false;
+
+#ifdef __linux__
+  if (executable_fd >= 0)
+    {
+      /* If we already have an executable_fd, check that filename points to
+	 the same inode.  */
+      struct stat statexe, statfile;
+
+      if (fstat (executable_fd, &statexe) < 0
+	  || stat (filename, &statfile) < 0
+	  || !(statfile.st_dev
+	       && statfile.st_dev == statexe.st_dev
+	       && statfile.st_ino == statexe.st_ino))
+	return false;
+
+      close (executable_fd);
+      executable_fd = -1;
+    }
+#endif
+
+  return true;
+}
+
+/* Determine the full pathname of the current executable, freshly allocated.
+   Return NULL if unknown.  Guaranteed to work on Linux and Win32, Mac OS X.
+   Likely to work on the other Unixes (maybe except BeOS), under most
+   conditions.  */
+char *
+_gst_find_executable (const char *argv0)
+{
+#if defined WIN32
+  char location[MAX_PATH];
+  int length = GetModuleFileName (NULL, location, sizeof (location));
+  if (length <= 0)
+    return NULL;
+
+#if defined __CYGWIN__
+  {
+    /* On Cygwin, we need to convert paths coming from Win32 system calls
+       to the Unix-like slashified notation.  */
+    static char location_as_posix_path[2 * MAX_PATH];
+
+    /* There's no error return defined for cygwin_conv_to_posix_path.
+       See cygwin-api/func-cygwin-conv-to-posix-path.html.
+       Does it overflow the buffer of expected size MAX_PATH or does it
+       truncate the path?  I don't know.  Let's catch both.  */
+    cygwin_conv_to_posix_path (location, location_as_posix_path);
+    location_as_posix_path[MAX_PATH - 1] = '\0';
+    if (strlen (location_as_posix_path) >= MAX_PATH - 1)
+      /* A sign of buffer overflow or path truncation.  */
+      return NULL;
+
+    return _gst_get_full_file_name (location_as_posix_path);
+  }
+#else
+  return xstrdup (location);
+#endif
+
+#else /* Unix && !Cygwin */
+#ifdef PATH_MAX
+  int path_max = PATH_MAX;
+#else
+  int path_max = pathconf (name, _PC_PATH_MAX);
+  if (path_max <= 0)
+    path_max = 1024;
+#endif
+
+#if HAVE_MACH_O_DYLD_H && HAVE__NSGETEXECUTABLEPATH
+  char *location = alloca (path_max);
+  uint32_t length = path_max;
+  if (_NSGetExecutablePath (location, &length) == 0 && location[0] == '/')
+    return _gst_get_full_file_name (location);
+
+#elif defined __linux__
+  /* The executable is accessible as /proc/<pid>/exe.  In newer Linux
+     versions, also as /proc/self/exe.  Linux >= 2.1 provides a symlink
+     to the true pathname; older Linux versions give only device and ino,
+     enclosed in brackets, which we cannot use here.  */
+  {
+    char buf[6 + 10 + 5];
+    char *location = alloca (path_max);
+    ssize_t n;
+
+    sprintf (buf, "/proc/%d/exe", getpid ());
+    n = readlink (buf, location, path_max);
+    if (n > 0 && location[0] != '[')
+      return location;
+    if (executable_fd < 0)
+      executable_fd = open (buf, O_RDONLY, 0);
+  }
+#endif
+
+  if (*argv0 == '-')
+    argv0++;
+
+  /* Guess the executable's full path.  We assume the executable has been
+     called via execlp() or execvp() with properly set up argv[0].
+     exec searches paths without slashes in the directory list given
+     by $PATH.  */
+  if (!strchr (argv0, '/'))
+    {
+      const char *p_next = getenv ("PATH");
+      const char *p;
+
+      while ((p = p_next) != NULL)
+	{
+	  char *concat_name;
+
+	  p_next = strchr (p, ':');
+	  /* An empty PATH element designates the current directory.  */
+	  if (p_next == p + 1)
+	    concat_name = xstrdup (argv0);
+	  else if (!p_next)
+	    asprintf (&concat_name, "%s/%s", p, argv0);
+	  else
+	    asprintf (&concat_name, "%.*s/%s", p_next++ - p, p, argv0);
+
+	  if (maybe_executable (concat_name))
+	    {
+	      char *full_path = _gst_get_full_file_name (concat_name);
+	      free (concat_name);
+	      return full_path;
+	    }
+
+	  free (concat_name);
+	}
+      /* Not found in the PATH, assume the current directory.  */
+    }
+
+  if (maybe_executable (argv0))
+    return _gst_get_full_file_name (argv0);
+
+  /* No way to find the executable.  */
+#ifdef __linux__
+  close (executable_fd);
+  executable_fd = -1;
+#endif
+  return NULL;
+#endif
+}
 
 
 /* Code to use PTY's did not work on Mac OS.  I'm keeping the Unix code,
@@ -1574,14 +1865,14 @@ _gst_debug (void)
 
 typedef struct heap_implementation {
   mst_Boolean (*check) ();
-  PTR (*reserve) (size_t);
+  PTR (*reserve) (PTR, size_t);
   void (*release) (PTR, size_t);
   PTR (*commit) (PTR, size_t);
   void (*decommit) (PTR, size_t);
 } heap_implementation;
 
 #ifdef WIN32
-static PTR win32_reserve (size_t);
+static PTR win32_reserve (PTR, size_t);
 static void win32_release (PTR, size_t);
 static PTR win32_commit (PTR, size_t);
 static void win32_decommit (PTR, size_t);
@@ -1591,12 +1882,15 @@ struct heap_implementation heap_impl_tab[] = {
 };
 #else /* !WIN32 */
 
+# if defined MAP_AUTORESRV && !defined MAP_NORESERVE
+#  define MAP_NORESERVE MAP_AUTORESRV
+# endif
 # ifdef MAP_NORESERVE
-static PTR noreserve_reserve (size_t);
+static PTR noreserve_reserve (PTR, size_t);
 static void noreserve_decommit (PTR, size_t);
 #endif
 static mst_Boolean anon_mmap_check (void);
-static PTR anon_mmap_reserve (size_t);
+static PTR anon_mmap_reserve (PTR, size_t);
 static void anon_mmap_release (PTR, size_t);
 static PTR anon_mmap_commit (PTR, size_t);
 
@@ -1626,7 +1920,7 @@ static int dev_zero = -1;
 static heap_implementation *impl;
 
 PTR
-_gst_osmem_reserve (size_t size)
+_gst_osmem_reserve (PTR address, size_t size)
 {
   if (!impl)
     {
@@ -1634,7 +1928,7 @@ _gst_osmem_reserve (size_t size)
          The check is done at run-time because it is cheap.  */
       for (impl = heap_impl_tab; impl->reserve; impl++)
         if (!impl->check || impl->check ())
-	  return impl->reserve (size);
+	  return impl->reserve (address, size);
 
       /* Not found, check again the next time just in case and return
          ENOMEM.  */
@@ -1643,7 +1937,7 @@ _gst_osmem_reserve (size_t size)
       return (NULL);
     }
   else
-    return impl->reserve (size);
+    return impl->reserve (address, size);
 }
 
 void
@@ -1708,10 +2002,12 @@ _gst_osmem_free (PTR ptr, size_t size)
 #ifdef WIN32
 
 PTR
-win32_reserve (size_t size)
+win32_reserve (PTR address, size_t size)
 {
   PTR base;
-  base = VirtualAlloc(NULL, size, MEM_RESERVE, PAGE_NOACCESS);
+  base = VirtualAlloc(address, size, MEM_RESERVE, PAGE_NOACCESS);
+  if (!base && address)
+    base = VirtualAlloc(NULL, size, MEM_RESERVE, PAGE_NOACCESS);
   if (!base)
     errno = ENOMEM;
 
@@ -1748,9 +2044,9 @@ win32_decommit (PTR base, size_t size)
    is available.  */
 
 PTR
-noreserve_reserve (size_t size)
+noreserve_reserve (PTR address, size_t size)
 {
-  PTR result = anon_mmap (NULL, size, PROT_NONE,
+  PTR result = anon_mmap (address, size, PROT_NONE,
 			  MAP_PRIVATE | MAP_NORESERVE);
 
   return result == MAP_FAILED ? NULL : result;
@@ -1771,11 +2067,11 @@ noreserve_decommit (PTR base, size_t size)
 static char *baseaddr;
 
 PTR
-anon_mmap_reserve (size_t size)
+anon_mmap_reserve (PTR address, size_t size)
 {
   PTR base;
 
-  /* We must check for overflows in baseaddr!  */
+  /* We must check for overflows in baseaddr!  Note that we ignore address.  */
   if (((uintptr_t) baseaddr) + size < (uintptr_t) baseaddr)
     {
       errno = ENOMEM;
