@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -138,12 +138,6 @@ extern time_t _gst_get_time (void)
 extern time_t _gst_get_file_modify_time (const char *fileName)
   ATTRIBUTE_HIDDEN;
 
-/* Sets the time when FILENAME was last modified.  The times are in
-   seconds, relative to 1 Jan 2000.  */
-extern int _gst_set_file_access_times (const char *name, long new_atime,
-				       long new_mtime)
-  ATTRIBUTE_HIDDEN;
-
 /* Converts the given time (expressed in seconds since midnight Jan 1,
    1970, and in Universal Coordinated Time) into a local time.  */
 extern time_t _gst_adjust_time_zone (time_t t)
@@ -249,7 +243,8 @@ extern void _gst_osmem_free (PTR ptr, size_t size)
   ATTRIBUTE_HIDDEN;
 
 /* Reserve SIZE bytes of the address space without allocating them.  */
-extern PTR _gst_osmem_reserve (size_t size)
+extern PTR _gst_osmem_reserve (PTR base,
+			       size_t size)
   ATTRIBUTE_HIDDEN;
 
 /* Release SIZE bytes of the address space starting from BASE.  */
