@@ -120,6 +120,7 @@ _gst_make_array_elt (YYLTYPE *location,
 
 tree_node
 _gst_make_method (YYLTYPE *location,
+		  YYLTYPE *endLocation,
 		  tree_node selectorExpr,
 		  tree_node temporaries,
 		  tree_node attributes,
@@ -128,6 +129,7 @@ _gst_make_method (YYLTYPE *location,
   tree_node result;
 
   result = make_tree_node (location, TREE_METHOD_NODE);
+  result->v_method.endPos = endLocation->file_offset;
   result->v_method.selectorExpr = selectorExpr;
   result->v_method.temporaries = temporaries;
   result->v_method.attributes = attributes;
@@ -455,14 +457,10 @@ _gst_make_binding_constant (YYLTYPE *location,
 		     tree_node variables)
 {
   tree_node result;
-  OOP assocOOP = _gst_find_variable_binding (variables);
-
-  if (IS_NIL (assocOOP))
-    return (NULL);
 
   result = make_tree_node (location, TREE_CONST_EXPR);
-  result->v_const.constType = CONST_OOP;
-  result->v_const.val.oopVal = assocOOP;
+  result->v_const.constType = CONST_BINDING;
+  result->v_const.val.aVal = variables;
 
   return (result);
 }
